--- namespace-definitions
local ClickReduce = {}
_G.ClickReduce = ClickReduce
local ClickReduceLocale = {}
_G.ClickReduceLocale = ClickReduceLocale
ClickReduceLocale.trans = {}

ClickReduce.details = {
	name = "ClickReduce",
	version = "0.5",
	author = "Jan Sandbrink",
	description = "Automatically skips unneccessary dialogs. Can also clean-up the UI.",
	category = "Other",
	configFrame = ClickReduceConfigFrame
}

--- Settings
ClickReduce.Settings = {
	skip_transport = true,
	skip_short_horse_rent = false,
	skip_horse_rent = true,
	skip_quest_complete = false,
	refuse_duel = false, ---決鬥
	class_change = false,---換職業
	equip_change = false,---切裝
	party_friends = true,---接受組隊
	auto_repair = true,---自動修裝
	
	override_warning_frame = false,
	warning_font_size = 15,
	warning_y_offset = 160,
	
	override_system_frame = false,
	system_font_size = 20,
	system_y_offset = 110,
	
	override_banner = false,
	banner_scale = 1.0,
	banner_y_offset = 140
}

--- registering events
local frame = _G.ClickReduceFrame
frame:RegisterEvent("VARIABLES_LOADED")
frame:RegisterEvent("SAVE_VARIABLES")
frame:RegisterEvent("DUEL_REQUESTED")
frame:RegisterEvent("SHOW_REQUEST_DIALOG")
frame:RegisterEvent("SHOW_QUESTDETAIL_FROM_NPC")
frame:RegisterEvent("EXCHANGECLASS_SHOW")
frame:RegisterEvent("EXCHANGECLASS_SUCCESS")
frame:RegisterEvent("PARTY_INVITE_REQUEST")
frame:RegisterEvent("STORE_OPEN")

local configFrame = _G.ClickReduceConfigFrame

--- make the slash-commands avaiable
SLASH_ClickReduce1 = "/clickreduce"
SLASH_ClickReduce2 = "/cr"
SlashCmdList["ClickReduce"] = function(editBox, msg)
			configFrame:Show()
end

-- load default language first
dofile("Interface/AddOns/ClickReduce/locales/en.lua")

-- load game-language in second step (if not fully translated, the rest will be english)
local lang = GetLanguage():upper()
local langname = "en"
if(lang == "TW") then
	langname = "tw"
elseif(lang == "CN") then
	langname = "cn"
end

-- only load second translation if it is not the default language
if(lang ~= "en") then
	dofile("Interface/AddOns/ClickReduce/locales/"..langname..".lua")
end

--- delegate all events to own handler-methods
function ClickReduce:OnEvent(event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
	self[event](self, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
end

function ClickReduce:VARIABLES_LOADED()
	-- register with AddonManager
	if AddonManager and AddonManager.RegisterAddonTable then
		AddonManager.RegisterAddonTable(ClickReduce.details)
    else
		DEFAULT_CHAT_FRAME:AddMessage(ClickReduceLocale.trans["startup_version"]..ClickReduce.details['version'])
		DEFAULT_CHAT_FRAME:AddMessage(ClickReduceLocale.trans["startup_usage"])
	end
	
	--load settings
	if(ClickReduce_Settings) then
	   for k,v in pairs(ClickReduce.Settings) do
		   if(ClickReduce_Settings[k] ~= nil) then
			   ClickReduce.Settings[k] = ClickReduce_Settings[k]
		   end
	   end
	end
	SaveVariables("ClickReduce_Settings")
	
	--customize UI
	ClickReduce.OverrideWorldFrames()
end

function ClickReduce:SAVE_VARIABLES()
	ClickReduce_Settings = ClickReduce.Settings;
end

function ClickReduce:DUEL_REQUESTED(player_name)
	if(ClickReduce.Settings["refuse_duel"]) then
		CancelDuel()
		StaticPopup1:Hide()
	end
end

function ClickReduce:STORE_OPEN()
	if(ClickReduce.Settings["auto_repair"]) then
		ClickRepairAllButton()
	end
end

function ClickReduce:SHOW_REQUEST_DIALOG(msg)
	if(ClickReduce.Settings["skip_transport"]) then
		if (string.find(msg, ClickReduceLocale.TransportMessage)) then
			ClickRequestDialogButton(0)
		end
	end
	
	if((ClickReduce.Settings["skip_short_horse_rent"] and string.find(msg, ClickReduceLocale.ShortRentalMessage))
		or (ClickReduce.Settings["skip_horse_rent"] and string.find(msg, ClickReduceLocale.RentalMessage))) then
		ClickRequestDialogButton(0)
	end
end

function ClickReduce:PARTY_INVITE_REQUEST(player_name)
	if(ClickReduce.Settings["party_friends"] and IsMyFriend(player_name)) then
		AcceptGroup()
		StaticPopup1:Hide()
	end
end

function ClickReduce:SHOW_QUESTDETAIL_FROM_NPC(quest_id)
	if(ClickReduce.Settings["skip_quest_complete"] and quest_id ~= nil) then
		if(ClickReduce:IsQuestComplete(quest_id)) then
			SpeakFrame_CompleteQuest()
			SpeakFrame_Hide()
		end
	end
end

function ClickReduce:EXCHANGECLASS_SHOW()
	if (ClickReduce.Settings["class_change"] and EXCHANGECLASS_SUBCLASS ~= 0) then
		ExchangeClass(EXCHANGECLASS_SUBCLASS, EXCHANGECLASS_MAINCLASS)
	end
end

function ClickReduce:EXCHANGECLASS_SUCCESS()
	if (ClickReduce.Settings["equip_change"]) then
		SwapEquipmentItem()
	end
end

function ClickReduce:IsQuestComplete(id)
	if(type(id) ~= "number") then
		return false
	else
		local i = 1
		local allCompleted = true
		repeat
			local item = GetQuestRequest(id, i)
			if(type(item) == "string") then
				if(not string.find(item, ClickReduceLocale.QuestComplete)) then
					allCompleted = false
					i = 0
				end
				i = i + 1
			else
				i = 0
			end
		until(i == 0)
		
		return allCompleted
	end
end

function ClickReduce.OverrideWorldFrames()
	if(ClickReduce.Settings['override_system_frame']) then
		SystemMsgFrame:ClearAllAnchors()
		SystemMsgFrame:SetAnchor("TOP", "TOP", WorldFrame, 0, ClickReduce.Settings['system_y_offset'])
		SystemMsgFrame:SetFontSize(ClickReduce.Settings['system_font_size'])
	end
	
	if(ClickReduce.Settings['override_warning_frame']) then
		WarningFrame:ClearAllAnchors()
		WarningFrame:SetAnchor("TOP", "TOP", WorldFrame, 0, ClickReduce.Settings['warning_y_offset'])
		WarningFrame:SetFontSize(ClickReduce.Settings['warning_font_size'])
	end
	
	if(ClickReduce.Settings['override_banner']) then
		ScrollBannerFrame:ClearAllAnchors()
		ScrollBannerFrame:SetAnchor("TOP", "TOP", WorldFrame, 0, ClickReduce.Settings['banner_y_offset'])
		ScrollBannerFrame:SetScale(ClickReduce.Settings['banner_scale'])
	end
end

function ClickReduce.TestSystemFrame()
	SystemMsgFrame:AddMessage(ClickReduceLocale.trans["test_message"])
end

function ClickReduce.TestWarningFrame()
	WarningFrame:AddMessage(ClickReduceLocale.trans["test_message"])
	WarningFrame:AddMessage(ClickReduceLocale.trans["test_message"])
	WarningFrame:AddMessage(ClickReduceLocale.trans["test_message"])
	WarningFrame:AddMessage(ClickReduceLocale.trans["test_message"])
end

function ClickReduce.TestScrollBanner()
	ScrollBannerFrame:Show()
end

function ClickReduce.OnShowConfig(source)
	getglobal(source:GetName().."_Title"):SetText(ClickReduceLocale.trans["config_title"])
	
	ClickReduceConfig_SkipTransport_Text:SetText(ClickReduceLocale.trans["config_skip_transport"])
	ClickReduceConfig_SkipShortHorseRent_Text:SetText(ClickReduceLocale.trans["config_skip_short_horse_rent"])
	ClickReduceConfig_SkipHorseRent_Text:SetText(ClickReduceLocale.trans["config_skip_horse_rent"])
--	ClickReduceConfig_SkipQuestComplete_Text:SetText(ClickReduceLocale.trans["config_skip_quest_complete"])
	ClickReduceConfig_RefuseDuel_Text:SetText(ClickReduceLocale.trans["config_refuse_duel"])
	ClickReduceConfig_ClassChange_Text:SetText(ClickReduceLocale.trans["config_class_change"])
	ClickReduceConfig_EquipChange_Text:SetText(ClickReduceLocale.trans["config_equip_change"])
	ClickReduceConfig_PartyFriends_Text:SetText(ClickReduceLocale.trans["config_party_friends"])
	ClickReduceConfig_AutoRepair_Text:SetText(ClickReduceLocale.trans["config_auto_repair"])
	
	ClickReduceConfig_OverrideSystemFrame_Text:SetText(ClickReduceLocale.trans["config_override_system_frame"])
	ClickReduceConfig_SystemFontSize_Text:SetText(ClickReduceLocale.trans["config_font_size"])
	ClickReduceConfig_TestSystemFrameButton:SetText(ClickReduceLocale.trans["config_test"])
	ClickReduceConfig_SystemYOffset_Text:SetText(ClickReduceLocale.trans["config_y_offset"])
	
	ClickReduceConfig_OverrideWarningFrame_Text:SetText(ClickReduceLocale.trans["config_override_warning_frame"])
	ClickReduceConfig_WarningFontSize_Text:SetText(ClickReduceLocale.trans["config_font_size"])
	ClickReduceConfig_TestWarningFrameButton:SetText(ClickReduceLocale.trans["config_test"])
	ClickReduceConfig_WarningYOffset_Text:SetText(ClickReduceLocale.trans["config_y_offset"])
	
	ClickReduceConfig_OverrideBanner_Text:SetText(ClickReduceLocale.trans["config_override_banner"])
	ClickReduceConfig_BannerScale_Text:SetText(ClickReduceLocale.trans["config_scale"])
	ClickReduceConfig_TestBannerButton:SetText(ClickReduceLocale.trans["config_test"])
	ClickReduceConfig_BannerYOffset_Text:SetText(ClickReduceLocale.trans["config_y_offset"])
	
	
	ClickReduceConfig_SkipTransport:SetChecked(ClickReduce.Settings['skip_transport'])
	ClickReduceConfig_SkipShortHorseRent:SetChecked(ClickReduce.Settings['skip_short_horse_rent'])	
	ClickReduceConfig_SkipHorseRent:SetChecked(ClickReduce.Settings['skip_horse_rent'])
--	ClickReduceConfig_SkipQuestComplete:SetChecked(ClickReduce.Settings['skip_quest_complete'])
	ClickReduceConfig_RefuseDuel:SetChecked(ClickReduce.Settings['refuse_duel'])
	ClickReduceConfig_ClassChange:SetChecked(ClickReduce.Settings['class_change'])
	ClickReduceConfig_EquipChange:SetChecked(ClickReduce.Settings['equip_change'])
	ClickReduceConfig_PartyFriends:SetChecked(ClickReduce.Settings['party_friends'])
	ClickReduceConfig_AutoRepair:SetChecked(ClickReduce.Settings['auto_repair'])
	
	ClickReduceConfig_OverrideSystemFrame:SetChecked(ClickReduce.Settings['override_system_frame'])
	ClickReduceConfig_SystemFontSize:SetText(ClickReduce.Settings['system_font_size'])
	ClickReduceConfig_SystemYOffset:SetText(ClickReduce.Settings['system_y_offset'])
	
	ClickReduceConfig_OverrideWarningFrame:SetChecked(ClickReduce.Settings['override_warning_frame'])
	ClickReduceConfig_WarningFontSize:SetText(ClickReduce.Settings['warning_font_size'])
	ClickReduceConfig_WarningYOffset:SetText(ClickReduce.Settings['warning_y_offset'])
	
	ClickReduceConfig_OverrideBanner:SetChecked(ClickReduce.Settings['override_banner'])
	ClickReduceConfig_BannerScale:SetText(ClickReduce.Settings['banner_scale']*100)
	ClickReduceConfig_BannerYOffset:SetText(ClickReduce.Settings['banner_y_offset'])
end

function ClickReduce.ConfigClick(source)
	local name = source:GetName();
	if(name == "ClickReduceConfig_SkipTransport") then
		ClickReduce.Settings["skip_transport"] = source:IsChecked()
	elseif(name == "ClickReduceConfig_SkipShortHorseRent") then
		ClickReduce.Settings["skip_short_horse_rent"] = source:IsChecked()
	elseif(name == "ClickReduceConfig_SkipHorseRent") then
		ClickReduce.Settings["skip_horse_rent"] = source:IsChecked()
	elseif(name == "ClickReduceConfig_SkipQuestComplete") then
		ClickReduce.Settings["skip_quest_complete"] = source:IsChecked()
	elseif(name == "ClickReduceConfig_RefuseDuel") then
		ClickReduce.Settings["refuse_duel"] = source:IsChecked()
	elseif(name == "ClickReduceConfig_ClassChange") then
		ClickReduce.Settings["class_change"] = source:IsChecked()
	elseif(name == "ClickReduceConfig_EquipChange") then
		ClickReduce.Settings["equip_change"] = source:IsChecked()
	elseif(name == "ClickReduceConfig_PartyFriends") then
		ClickReduce.Settings["party_friends"] = source:IsChecked()
	elseif(name == "ClickReduceConfig_AutoRepair") then
		ClickReduce.Settings["auto_repair"] = source:IsChecked()
	elseif(name == "ClickReduceConfig_OverrideSystemFrame") then
		ClickReduce.Settings["override_system_frame"] = source:IsChecked()
	elseif(name == "ClickReduceConfig_OverrideWarningFrame") then
		ClickReduce.Settings["override_warning_frame"] = source:IsChecked()
	elseif(name == "ClickReduceConfig_OverrideBanner") then
		ClickReduce.Settings["override_banner"] = source:IsChecked()
	end
end

function ClickReduce.SaveConfigBoxes()
	-- settings for SystemMsgFrame
	if(#ClickReduceConfig_SystemFontSize:GetText() > 0) then
		ClickReduce.Settings['system_font_size'] = tonumber(ClickReduceConfig_SystemFontSize:GetText())
	end
	if(#ClickReduceConfig_SystemYOffset:GetText() > 0) then
		ClickReduce.Settings['system_y_offset'] = tonumber(ClickReduceConfig_SystemYOffset:GetText())
	end
	
	-- settings for WarningFrame
	if(#ClickReduceConfig_WarningFontSize:GetText() > 0) then
		ClickReduce.Settings['warning_font_size'] = tonumber(ClickReduceConfig_WarningFontSize:GetText())
	end
	if(#ClickReduceConfig_WarningYOffset:GetText() > 0) then
		ClickReduce.Settings['warning_y_offset'] = tonumber(ClickReduceConfig_WarningYOffset:GetText())
	end
	
	-- settings for ScrollBannerFrame
	if(#ClickReduceConfig_BannerScale:GetText() > 0) then
		ClickReduce.Settings['banner_scale'] = tonumber(ClickReduceConfig_BannerScale:GetText())/100
	end
	if(#ClickReduceConfig_BannerYOffset:GetText() > 0) then
		ClickReduce.Settings['banner_y_offset'] = tonumber(ClickReduceConfig_BannerYOffset:GetText())
	end
	
	-- execute changes
	ClickReduce.OverrideWorldFrames()
end