-- (C) 2012-2013 Miranda
--------------------------------------------------------------------------------
-- 名前空間作成
if(_G and not _G.Kitty) then
	_G.Kitty = {};
end

--------------------------------------------------------------------------------
Kitty.CommandConditionFrame = {};

-- 編集前のコマンドリスト
Kitty.CommandConditionFrame.commandList = nil;
-- 編集対象のコマンドのインデックス
Kitty.CommandConditionFrame.targetCommandIndex = nil;
-- 編集対象のコマンド
Kitty.CommandConditionFrame.targetConditionList = nil;
-- 親画面表示関数
Kitty.CommandConditionFrame.callbackFunc = nil;

-- UIコントロール/データ対応表
Kitty.CommandConditionFrame.relativeList = {
	-- 静的メッセージ一覧
	["KittyCommandConditionFrame_VersionLabel"] = "KittyCombo Version:[" .. Kitty.VERSION .. "]",
	["KittyCommandConditionFrame_TitleLabel"] = Kitty.T("message", "ConditionTitle"),
	["KittyCommandConditionFrame_CommandListLabel"] = Kitty.T("message", "CommandListNameLabel") .. ":",
	["KittyCommandConditionFrame_BuildLabel"] = Kitty.T("message", "BuildLabel") .. ":",
	["KittyCommandConditionFrame_TargetLabel"] = Kitty.T("message", "CommandListTargetLabel") .. ":",
	["KittyCommandConditionFrame_CommandLabel"] = Kitty.T("message", "CommandLabel") .. ":",

	-- 動的メッセージ一覧
	["KittyCommandConditionFrame_CommandListNameLabel"]	= "name",
	["KittyCommandConditionFrame_BuildNameLabel"] = "class",
	["KittyCommandConditionFrame_TargetNameLabel"] = "target",
	["KittyCommandConditionFrame_CommandNameLabel"] = "",

	["KittyCommandConditionFrame_CloseButton"] = Kitty.T("message", "CloseButton"),
	["KittyCommandConditionFrame_CancelButton"] = Kitty.T("message", "CancelButton"),
};

--------------------------------------------------------------------------------
Kitty.CommandConditionFrame.toString = function(command)
	local str = "";

	str = str .. "CommandType:[" .. tostring(command.commandType) .. "], CommandName:[" .. tostring(command.commandName) .. "], Enable:[" .. tostring(command.enable) .. "]\n";
	for index, condition in ipairs(command) do
		if(condition.conditionType == nil) then
			str = "ConditionType:[" .. tostring(condition.conditionType) .. "]";
		else
			local formatType = Kitty.ComboManager.getFormatType(condition.conditionType);
			if(formatType == Kitty.ComboManager.TYPE_STRING_NUMBER_LIST or formatType == Kitty.ComboManager.TYPE_EFFECT) then
				for index, effect in ipairs(condition) do
					str = str .. "ConditionType:[" .. tostring(condition.conditionType) .. "], EffectType:[" .. tostring(condition.effectType) .. "], Name:[" .. tostring(condition.name) .. "], Threshold:[" .. tostring(condition.threshold) .. "]\n";
				end
			else
				str = str .. "ConditionType:[" .. tostring(condition.conditionType) .. "], Min:[" .. tostring(condition.min) .. "], Max:[" .. tostring(condition.max) .. "], Value:[" .. tostring(condition.value) .. "]\n";
			end
		end
	end
	return str;
end;

--[[
- 入力
command = {
	["commandType"]="extra", ["commandName"]="name",
	{["conditionType"]="playerEffect",	{["name"]="1st"},
										{["name"]="2nd"}},
	{["conditionType"]="targetEffect"},
};
- 出力
command = {
	["commandType"]="extra", ["commandName"]="name",
	{["conditionType"]="playerEffect", ["name"]="1st"},
	{["conditionType"]="playerEffect", ["name"]="2nd"}
	{["conditionType"]="targetEffect"},
};
--]]
-- 分離
Kitty.CommandConditionFrame.separate = function(command)
	command = Kitty.copyTable(command);

	local resultArray = {};
	for key, condition in pairs(command) do
		if(type(key)=="number") then
			-- Keyが数値のデータ
			if(type(condition)=="table") then
				-- 要素はテーブル型
				local formatType = Kitty.ComboManager.getFormatType(condition.conditionType);
				if(formatType == Kitty.ComboManager.TYPE_STRING_NUMBER_LIST
				or formatType == Kitty.ComboManager.TYPE_EFFECT) then
					-- 分離必要
					for itemKey, item in ipairs(condition) do
						if(type(itemKey)=="number") then
							if(type(item)=="table") then
								item.conditionType = condition.conditionType;
								table.insert(resultArray, item);
							else
								-- Negrect.
							end
						else
							if(key~="conditionType") then
								-- Negrect.
							end
						end
					end
				else
					-- 分離不要
					table.insert(resultArray, condition);
				end
			else
				-- 要素はテーブル型以外
				table.insert(resultArray, condition);
			end
		else
			-- Keyが数値以外のデータ
			resultArray[key] = condition;
		end
	end
	return resultArray;
end;
-- マージ
Kitty.CommandConditionFrame.merge = function(command)
	command = Kitty.copyTable(command);

	local resultArray = {};
	local mergeList = {};
	for key, condition in pairs(command) do
		if(type(key)=="number") then
			-- Keyが数値のデータ
			if(type(condition)=="table") then
				-- 要素はテーブル型
				local formatType = Kitty.ComboManager.getFormatType(condition.conditionType);
				if(formatType == Kitty.ComboManager.TYPE_STRING_NUMBER_LIST
				or formatType == Kitty.ComboManager.TYPE_EFFECT) then
					-- マージ必要
					if(mergeList[condition.conditionType] == nil) then
						-- 初回
						local newList = {};
						newList.conditionType = condition.conditionType;
						table.insert(newList, condition);

						mergeList[condition.conditionType] = newList;
						condition.conditionType = nil;
						table.insert(resultArray, newList);
					else
						-- 初回以降
						table.insert(mergeList[condition.conditionType], condition);
						condition.conditionType = nil;
					end
				else
					-- マージ不要
					table.insert(resultArray, condition);
				end
			else
				-- 要素はテーブル型以外
				table.insert(resultArray, condition);
			end
		else
			-- Keyが数値以外のデータ
			resultArray[key] = condition;
		end
	end
	return resultArray;
end;

--------------------------------------------------------------------------------
-- 画面表示
Kitty.CommandConditionFrame.open = function(commandList, index, callbackFunc)
	Kitty.CommandConditionFrame.callbackFunc = callbackFunc;

	Kitty.CommandConditionFrame.commandList = commandList;
	Kitty.CommandConditionFrame.targetCommandIndex = index;
	local conditionList = Kitty.CommandConditionFrame.commandList[Kitty.CommandConditionFrame.targetCommandIndex];
	Kitty.CommandConditionFrame.targetConditionList = Kitty.ListManager(Kitty.copyTable(Kitty.CommandConditionFrame.separate(conditionList)));

	if(#Kitty.CommandConditionFrame.targetConditionList == 0) then
		-- Create Empty
		table.insert(Kitty.CommandConditionFrame.targetConditionList, {["conditionType"]=""});
	end

	Kitty.CommandConditionFrame.relativeList["KittyCommandConditionFrame_CommandNameLabel"] = conditionList.commandName;
	Kitty.settingToComponent(commandList.info, Kitty.CommandConditionFrame.relativeList);

	-- 明細表示
	Kitty.CommandConditionFrame.onRedraw(this, 1, 10);

	KittyCommandConditionFrame:Show();
end;

Kitty.CommandConditionFrame.save = function()
	-- マージ
	local mergedList = Kitty.CommandConditionFrame.merge(Kitty.CommandConditionFrame.targetConditionList);
	Kitty.CommandConditionFrame.commandList[Kitty.CommandConditionFrame.targetCommandIndex] = mergedList;

	Kitty.debugPrint("mergedList", mergedList);
end;
Kitty.CommandConditionFrame.close = function()
	Kitty.CommandConditionFrame.commandList = nil;
	Kitty.CommandConditionFrame.targetCommandIndex = nil;
	Kitty.CommandConditionFrame.targetConditionList = nil;
	Kitty.CommandConditionFrame.callbackFunc = nil;

	KittyCommandConditionFrame:Hide();
end;

-- リストの明細行に表示されている項目を内部データへ格納
Kitty.CommandConditionFrame.allowUpdateData = true;
Kitty.CommandConditionFrame.updateData = function(this)
	if(Kitty.CommandConditionFrame.allowUpdateData) then
		Kitty.debugPrint("Kitty.CommandConditionFrame.updateData");
	else
		Kitty.debugPrint("Kitty.CommandConditionFrame.updateData Canceled.");
		return;
	end
	-- ドロップダウンメニュー(コマンド発動条件種別)
	local dropDownMenu = getglobal(this:GetName() .. "DropDownMenu");
	assert(dropDownMenu, this:GetName() .. "DropDownMenu is nil");

	-- ドロップダウンメニュー(Efect指定種別)
	local effectTypeDropDownMenu = getglobal(this:GetName() .. "EffectTypeDropDownMenu");
	assert(effectTypeDropDownMenu, this:GetName() .. "EffectTypeDropDownMenu is nil");
	-- ドロップダウンメニュー(バフ種別)
	local conditionDropDownMenu = getglobal(this:GetName() .. "ConditionDropDownMenu");
	assert(conditionDropDownMenu, this:GetName() .. "ConditionDropDownMenu is nil");
	-- エディットボックス(設定値用)
	local conditionEditBox = getglobal(this:GetName() .. "ConditionValueEditBox");
	assert(conditionEditBox, this:GetName() .. "ConditionValueEditBox is nil");
	-- エディットボックス(閾値用)
	local thresholdEditBox = getglobal(this:GetName() .. "ConditionThresholdEditBox");
	assert(thresholdEditBox, this:GetName() .. "ConditionThresholdEditBox is nil");
	-- エディットボックス(最小値用)
	local minEditBox = getglobal(this:GetName() .. "MinEditBox");
	assert(minEditBox, this:GetName() .. "MinEditBox is nil");
	-- エディットボックス(最大値用)
	local maxEditBox = getglobal(this:GetName() .. "MaxEditBox");
	assert(maxEditBox, this:GetName() .. "MaxEditBox is nil");
	-- ドロップダウンメニュー(有無設定)
	local booleanDropDownMenu = getglobal(this:GetName() .. "BooleanDropDownMenu");
	assert(booleanDropDownMenu, this:GetName() .. "BooleanDropDownMenu is nil");
	-- ドロップダウンメニュー(ショートカット選択)
	local actionDropDownMenu = getglobal(this:GetName() .. "ActionDropDownMenu");
	assert(actionDropDownMenu, this:GetName() .. "ActionDropDownMenu is nil");
	-- ドロップダウンメニュー(ビルド選択)
	local buildDropDownMenu = getglobal(this:GetName() .. "BuildDropDownMenu");
	assert(buildDropDownMenu, this:GetName() .. "BuildDropDownMenu is nil");

	-- ドロップダウンメニュー(コマンド発動条件種別)
	local condition = {};
	condition.conditionType = UIDropDownMenu_GetText(dropDownMenu);
	if(condition.conditionType ~= nil and condition.conditionType ~= "") then
		local formatType = Kitty.ComboManager.getFormatType(condition.conditionType);

		if(formatType == Kitty.ComboManager.TYPE_EFFECT) then
			-- ドロップダウンメニュー(Efect指定種別)
			condition.effectType = UIDropDownMenu_GetText(effectTypeDropDownMenu);
			-- エディットボックス(設定値用)
			if(condition.effectType == "original") then
				condition.name = UIDropDownMenu_GetText(conditionDropDownMenu);
			else
				condition.name = conditionEditBox:GetText();
			end
			-- エディットボックス(閾値用)
			condition.threshold = tonumber(thresholdEditBox:GetText());
			if(condition.threshold == nil) then
				condition.threshold = 0;
			end
		elseif(formatType == Kitty.ComboManager.TYPE_ACTION) then
			condition.value = UIDropDownMenu_GetText(actionDropDownMenu);
		elseif(formatType == Kitty.ComboManager.TYPE_BUILD) then
			condition.value = UIDropDownMenu_GetText(buildDropDownMenu);
		elseif(formatType == Kitty.ComboManager.TYPE_STRING_NUMBER_LIST) then
			-- エディットボックス(設定値用)
			condition.name = conditionEditBox:GetText();
			-- エディットボックス(閾値用)
			condition.threshold = tonumber(thresholdEditBox:GetText());
			if(condition.threshold == nil) then
				condition.threshold = 0;
			end
		elseif(formatType == Kitty.ComboManager.TYPE_STRING) then
			-- エディットボックス(設定値用)
			condition.value = conditionEditBox:GetText();
		elseif(formatType == Kitty.ComboManager.TYPE_NUMBER) then
			-- エディットボックス(最小値用/最大値用)
			condition.min = tonumber(minEditBox:GetText());
			condition.max = tonumber(maxEditBox:GetText());
		elseif(formatType == Kitty.ComboManager.TYPE_BOOLEAN) then
			-- ドロップダウンメニュー(有無設定)
			if(UIDropDownMenu_GetText(booleanDropDownMenu) == "true") then
				condition.value = true;
			else
				condition.value = false;
			end
		else
			error("Invalid DataType:[" .. tostring(formatType) .. "]");
		end
	end

	local index = KittyCommandConditionFrame_ScrollBar:GetValue() + this:GetID() - 1;
	Kitty.CommandConditionFrame.targetConditionList[index] = condition;
end;

-- コマンドリスト一覧の明細表示
Kitty.CommandConditionFrame.onRedraw = function(this, start, count)
	local position = KittyCommandConditionFrame_ScrollBar:GetValue();
	if(start == position) then
		-- 再描画
		Kitty.CommandConditionFrame.onShowList(KittyCommandConditionFrame_ScrollBar, start, count);
	else
		-- スクロールバー更新(その後再描画される)
		KittyCommandConditionFrame_ScrollBar:SetValue(start);
	end
end;
Kitty.CommandConditionFrame.onShowList = function(this, start, count)
	Kitty.CommandConditionFrame.allowUpdateData = false;
	Kitty.debugPrint("Kitty.CommandConditionFrame.onShowList");

	if(count ~= nil) then
		count = 10;
	end

	-- スライダーの範囲を確定
	local aboveRange = start - 1;	-- 上方へスクロール可能なコマ数
	if(aboveRange < 0) then
		aboveRange = 0;
	end
	local belowRange = 0;
	if(Kitty.CommandConditionFrame.targetConditionList ~= nil) then
		belowRange = #Kitty.CommandConditionFrame.targetConditionList - count - (start - 1);	-- 下方へスクロール可能なコマ数
	end
	if(belowRange < 0) then
		belowRange = 0;
	end
	local range = aboveRange + belowRange + 1;

	if(range <= 0) then
		-- スライダー不要
		this:SetMinMaxValues(1, 1);
	else
		this:SetMinMaxValues(1, range);
	end

	-- Debug
	local min, max = this:GetMinMaxValues();
	Kitty.debugPrint("Scroll Value:[" .. tostring(this:GetValue()) .. "], Range:[" .. min .. ", " .. max .. "], Above:[" .. aboveRange .. "], Below:[" .. belowRange .. "]");

	-- 明細を表示
	local position = 1;
	for index=start, start+count-1 do
		local detailButton = getglobal("KittyCommandConditionFrame_Button" .. tostring(position));
		assert(detailButton, "KittyCommandConditionFrame_Button" .. tostring(position) .. " is nil.");

		-- 明細更新
		if(Kitty.CommandConditionFrame.targetConditionList ~= nil) then
			local condition = Kitty.CommandConditionFrame.targetConditionList[index];
			if(condition ~= nil) then
				Kitty.CommandConditionFrame.showDetail(detailButton, condition);
				detailButton:Show();
			else
				detailButton:Hide();
			end
		end

		position = position + 1;
	end

	Kitty.CommandConditionFrame.allowUpdateData = true;
end;

-- 内部データをリストの明細行へ表示
Kitty.CommandConditionFrame.showDetail = function(this, condition)
	-- ドロップダウンメニュー(コマンド発動条件種別)
	local dropDownMenu = getglobal(this:GetName() .. "DropDownMenu");
	assert(dropDownMenu, this:GetName() .. "DropDownMenu is nil");

	-- ドロップダウンメニュー(Efect指定種別)
	local effectTypeDropDownMenu = getglobal(this:GetName() .. "EffectTypeDropDownMenu");
	assert(effectTypeDropDownMenu, this:GetName() .. "EffectTypeDropDownMenu is nil");
	-- ドロップダウンメニュー(バフ種別)
	local conditionDropDownMenu = getglobal(this:GetName() .. "ConditionDropDownMenu");
	assert(conditionDropDownMenu, this:GetName() .. "ConditionDropDownMenu is nil");
	-- エディットボックス(設定値用)
	local conditionEditBox = getglobal(this:GetName() .. "ConditionValueEditBox");
	assert(conditionEditBox, this:GetName() .. "ConditionValueEditBox is nil");
	-- エディットボックス(閾値用)
	local thresholdEditBox = getglobal(this:GetName() .. "ConditionThresholdEditBox");
	assert(thresholdEditBox, this:GetName() .. "ConditionThresholdEditBox is nil");
	-- エディットボックス(最小値用)
	local minEditBox = getglobal(this:GetName() .. "MinEditBox");
	assert(minEditBox, this:GetName() .. "MinEditBox is nil");
	-- エディットボックス(最大値用)
	local maxEditBox = getglobal(this:GetName() .. "MaxEditBox");
	assert(maxEditBox, this:GetName() .. "MaxEditBox is nil");
	-- ドロップダウンメニュー(有無設定)
	local booleanDropDownMenu = getglobal(this:GetName() .. "BooleanDropDownMenu");
	assert(booleanDropDownMenu, this:GetName() .. "BooleanDropDownMenu is nil");
	-- ドロップダウンメニュー(ショートカット選択)
	local actionDropDownMenu = getglobal(this:GetName() .. "ActionDropDownMenu");
	assert(actionDropDownMenu, this:GetName() .. "ActionDropDownMenu is nil");
	-- ドロップダウンメニュー(ビルド選択)
	local buildDropDownMenu = getglobal(this:GetName() .. "BuildDropDownMenu");
	assert(buildDropDownMenu, this:GetName() .. "BuildDropDownMenu is nil");

	UIDropDownMenu_SetText(dropDownMenu, tostring(condition.conditionType));
	-- 発動条件により、入力欄の種類を変更
	local formatType = nil
	if(condition.conditionType ~= nil and condition.conditionType ~= "") then
		formatType = Kitty.ComboManager.getFormatType(condition.conditionType);
	else
		formatType = Kitty.ComboManager.TYPE_BOOLEAN;
	end

	-- 初期化
	UIDropDownMenu_SetText(effectTypeDropDownMenu, "");
	conditionEditBox:SetText("");
	thresholdEditBox:SetText("");
	minEditBox:SetText("");
	maxEditBox:SetText("");
	UIDropDownMenu_SetText(booleanDropDownMenu, "");

	if(formatType == Kitty.ComboManager.TYPE_EFFECT) then
		if(condition.effectType == nil) then
			condition.effectType = "name";
		end
		UIDropDownMenu_SetText(effectTypeDropDownMenu, tostring(condition.effectType));

		-- バフ指定方法で場合分け
		if(condition.effectType == "original") then
			if(condition.name == nil) then
				condition.name = "";
			end
			UIDropDownMenu_SetText(conditionDropDownMenu, tostring(condition.name));

			if(condition.threshold == nil) then
				condition.threshold = 0.0;
			end
			thresholdEditBox:SetText(tostring(condition.threshold));

			effectTypeDropDownMenu:Show();
			conditionDropDownMenu:Show();
			conditionEditBox:Hide();
			thresholdEditBox:Show();
			minEditBox:Hide();
			maxEditBox:Hide();
			booleanDropDownMenu:Hide();
			actionDropDownMenu:Hide();
			buildDropDownMenu:Hide();
		else
			if(condition.name == nil) then
				condition.name = "";
			end
			conditionEditBox:SetText(tostring(condition.name));

			if(condition.threshold == nil) then
				condition.threshold = 0.0;
			end
			thresholdEditBox:SetText(tostring(condition.threshold));

			effectTypeDropDownMenu:Show();
			conditionDropDownMenu:Hide();
			conditionEditBox:Show();
			thresholdEditBox:Show();
			minEditBox:Hide();
			maxEditBox:Hide();
			booleanDropDownMenu:Hide();
			actionDropDownMenu:Hide();
			buildDropDownMenu:Hide();
		end
	elseif(formatType == Kitty.ComboManager.TYPE_ACTION) then
		if(condition.value == nil) then
			condition.value = "";
		end
		UIDropDownMenu_SetText(actionDropDownMenu, tostring(condition.value));

		effectTypeDropDownMenu:Hide();
		conditionDropDownMenu:Hide();
		conditionEditBox:Hide();
		thresholdEditBox:Hide();
		minEditBox:Hide();
		maxEditBox:Hide();
		booleanDropDownMenu:Hide();
		actionDropDownMenu:Show();
		buildDropDownMenu:Hide();
	elseif(formatType == Kitty.ComboManager.TYPE_BUILD) then
		if(condition.value == nil) then
			condition.value = "";
		end
		UIDropDownMenu_SetText(buildDropDownMenu, tostring(condition.value));

		effectTypeDropDownMenu:Hide();
		conditionDropDownMenu:Hide();
		conditionEditBox:Hide();
		thresholdEditBox:Hide();
		minEditBox:Hide();
		maxEditBox:Hide();
		booleanDropDownMenu:Hide();
		actionDropDownMenu:Hide();
		buildDropDownMenu:Show();
	elseif(formatType == Kitty.ComboManager.TYPE_STRING_NUMBER_LIST) then
		if(condition.name == nil) then
			condition.name = "";
		end
		conditionEditBox:SetText(tostring(condition.name));

		if(condition.threshold == nil) then
			condition.threshold = 0.0;
		end
		thresholdEditBox:SetText(tostring(condition.threshold));

		effectTypeDropDownMenu:Hide();
		conditionDropDownMenu:Hide();
		conditionEditBox:Show();
		thresholdEditBox:Show();
		minEditBox:Hide();
		maxEditBox:Hide();
		booleanDropDownMenu:Hide();
		actionDropDownMenu:Hide();
		buildDropDownMenu:Hide();
	elseif(formatType == Kitty.ComboManager.TYPE_STRING) then
		if(condition.value == nil) then
			condition.value = "";
		end
		conditionEditBox:SetText(tostring(condition.value));

		effectTypeDropDownMenu:Hide();
		conditionDropDownMenu:Hide();
		conditionEditBox:Show();
		thresholdEditBox:Hide();
		minEditBox:Hide();
		maxEditBox:Hide();
		booleanDropDownMenu:Hide();
		actionDropDownMenu:Hide();
		buildDropDownMenu:Hide();
	elseif(formatType == Kitty.ComboManager.TYPE_NUMBER) then
		if(condition.min == nil) then
			condition.min = 0;
			minEditBox:SetText(tostring(condition.min));
		end
		minEditBox:SetText(tostring(condition.min));

		if(condition.max == nil) then
			condition.max = 100;
		end
		maxEditBox:SetText(tostring(condition.max));

		effectTypeDropDownMenu:Hide();
		conditionDropDownMenu:Hide();
		conditionEditBox:Hide();
		thresholdEditBox:Hide();
		minEditBox:Show();
		maxEditBox:Show();
		booleanDropDownMenu:Hide();
		actionDropDownMenu:Hide();
		buildDropDownMenu:Hide();
	elseif(formatType == Kitty.ComboManager.TYPE_BOOLEAN) then
		if(condition.value == nil) then
			condition.value = true;
		end
		UIDropDownMenu_SetText(booleanDropDownMenu, tostring(condition.value));

		effectTypeDropDownMenu:Hide();
		conditionDropDownMenu:Hide();
		conditionEditBox:Hide();
		thresholdEditBox:Hide();
		minEditBox:Hide();
		maxEditBox:Hide();
		booleanDropDownMenu:Show();
		actionDropDownMenu:Hide();
		buildDropDownMenu:Hide();
	else
		error("Invalid DataType:[" .. tostring(formatType) .. "]");
	end
end;
