-- (C) 2012-2013 Miranda
--------------------------------------------------------------------------------
-- 名前空間作成
if(_G and not _G.Kitty) then
	_G.Kitty = {};
end

--------------------------------------------------------------------------------
Kitty.CommandConfigFrame = {};
-- 編集前のコマンドリスト群
Kitty.CommandConfigFrame.allCommandList = nil;
-- 編集対象のコマンドリストのインデックス
Kitty.CommandConfigFrame.targetCommandListIndex = nil;
-- 編集対象のコマンドリスト
Kitty.CommandConfigFrame.targetCommandList = nil;
-- 親画面表示関数
Kitty.CommandConfigFrame.callbackFunc = nil;

Kitty.CommandConfigFrame.dragAndDropManager = nil;

-- UIコントロール/データ対応表
Kitty.CommandConfigFrame.relativeList = {
	-- 静的メッセージ一覧
	["KittyCommandConfigFrame_VersionLabel"] = "KittyCombo Version:[" .. Kitty.VERSION .. "]",
	["KittyCommandConfigFrame_TitleLabel"] = Kitty.T("message", "CommandListTitle"),
	["KittyCommandConfigFrame_CommandListNameLabel"] = Kitty.T("message", "CommandListNameLabel"),
	["KittyCommandConfigFrame_BuildLabel"] = Kitty.T("message", "BuildLabel"),
	["KittyCommandConfigFrame_TargetLabel"] = Kitty.T("message", "CommandListTargetLabel"),

	["KittyCommandConfigFrame_CloseButton"] = Kitty.T("message", "CloseButton"),
	["KittyCommandConfigFrame_CancelButton"] = Kitty.T("message", "CancelButton"),

	-- 動的メッセージ一覧
	["KittyCommandConfigFrame_CommandListNameEditBox"] = "name",
	["KittyCommandConfigFrame_BuildDropDownMenu"] = "class",
	["KittyCommandConfigFrame_TargetDropDownMenu"] = "target",
};

-- ユニーク名を作成
Kitty.CommandConfigFrame.createUniqueName = function(base, list)
	local index = 0;
	local name = nil;
	repeat
		index = index + 1;
		name = base .. string.format("%03d", index);
	until(list[name] == nil);

	return name;
end;

--------------------------------------------------------------------------------
-- 画面表示
Kitty.CommandConfigFrame.open = function(allCommandList, index, callbackFunc)
	Kitty.CommandConfigFrame.callbackFunc = callbackFunc;

	Kitty.CommandConfigFrame.allCommandList = allCommandList;
	Kitty.CommandConfigFrame.targetCommandListIndex = index;

	-- コマンドリスト一覧画面からの遷移
	if(Kitty.CommandConfigFrame.targetCommandListIndex > 0) then
		-- DeepCopy
		Kitty.CommandConfigFrame.targetCommandList = Kitty.ListManager(Kitty.copyTable(Kitty.CommandConfigFrame.allCommandList[Kitty.CommandConfigFrame.targetCommandListIndex]));
	else
		-- コマンドリスト名称の作成
		commandListNameList = {};
		for index, commandList in pairs(allCommandList) do
			if(commandList.info ~= nil) then
				commandListNameList[commandList.info.name] = true;
			end
		end
		local _, _, main, sub = Kitty.BuildManager.getClass("player");
		local commandListName = Kitty.CommandConfigFrame.createUniqueName(main .. sub .. "_", commandListNameList);

		-- 新規作成
		local command = {["info"] = {["name"]=commandListName, ["class"]=main .. sub, ["target"]="all"}, {["commandType"]="", ["commandName"]="", ["icon"]="", ["enable"]=true}};
		Kitty.CommandConfigFrame.targetCommandList = Kitty.ListManager(command);
	end

	Kitty.settingToComponent(Kitty.CommandConfigFrame.targetCommandList.info, Kitty.CommandConfigFrame.relativeList);

	-- 明細表示
	Kitty.CommandConfigFrame.onRedraw(this, 1, 10);

	KittyCommandConfigFrame:Show();
end;
Kitty.CommandConfigFrame.update = function(commandList)
	Kitty.CommandConfigFrame.targetCommandList = commandList;

	-- 明細表示
	Kitty.CommandConfigFrame.onRedraw(this, 1, 10);

	KittyCommandConfigFrame:Show();
end;

Kitty.CommandConfigFrame.save = function()
	Kitty.componentToSetting(Kitty.CommandConfigFrame.targetCommandList.info, Kitty.CommandConfigFrame.relativeList);

	if(Kitty.CommandConfigFrame.targetCommandListIndex > 0) then
		-- 更新
		Kitty.CommandConfigFrame.allCommandList[Kitty.CommandConfigFrame.targetCommandListIndex] = Kitty.CommandConfigFrame.targetCommandList;
	else
		-- 新規作成
		table.insert(Kitty.CommandConfigFrame.allCommandList, 1, Kitty.CommandConfigFrame.targetCommandList);
	end

	-- 設定変更イベント通知
--	Kitty.onVariablesChanged(Kitty.SettingManager.setting);
end;
Kitty.CommandConfigFrame.close = function()
	Kitty.CommandConfigFrame.allCommandList = nil;
	Kitty.CommandConfigFrame.targetCommandListIndex = nil;
	Kitty.CommandConfigFrame.targetCommandList = nil;
	Kitty.CommandConfigFrame.callbackFunc = nil;

	KittyCommandConfigFrame:Hide();
end;

-- クリック時の処理
Kitty.CommandConfigFrame.onClick = function(this)
	local index = KittyCommandConfigFrame_ScrollBar:GetValue() + this:GetParent():GetID() - 1;
	KittyCommandConfigFrame:Hide();
	Kitty.CommandConditionFrame.open(Kitty.CommandConfigFrame.targetCommandList, index, Kitty.CommandConfigFrame.update);
end;

Kitty.CommandConfigFrame.onShow = function(this)
	-- Hook開始
	if(Kitty.CommandConfigFrame.dragAndDropManager == nil) then
		Kitty.CommandConfigFrame.dragAndDropManager = Kitty.DragAndDropManager();
	end
	Kitty.CommandConfigFrame.dragAndDropManager:startHook();
end;
Kitty.CommandConfigFrame.onHide = function(this)
	-- Hook終了
	if(Kitty.CommandConfigFrame.dragAndDropManager ~= nil) then
		Kitty.CommandConfigFrame.dragAndDropManager:endHook()
	end
end;

-- Drop
Kitty.CommandConfigFrame.onDrop = function(this)
	-- ドロップアイコン情報取得
	local cursorType, index1, index2 = Kitty.CommandConfigFrame.dragAndDropManager:getInfo();
	Kitty.CommandConfigFrame.dragAndDropManager:clear();

	if(cursorType ~= CursorItemType()) then
		-- ドラッグ開始時と現在のカーソルタイプが異なる
		return;
	end

	-- SkillManager
	local skillManager = Kitty.SkillManager(Kitty.cooldownManager);
	skillManager:update();

	-- コマンド取得
	local index = KittyCommandConfigFrame_ScrollBar:GetValue() + this:GetParent():GetID() - 1;
	local command = Kitty.CommandConfigFrame.targetCommandList[index];

	-- ドロップした種類ごとに処理
	if(cursorType == Kitty.DragAndDropManager.SKILL) then
		--------------------------------------------------------------------------------
		-- スキル
		ItemClipboard_Clear();	-- カーソルクリア

		local skillName, _, icon, skillType, rank, _, upgradeCost, isSkillable, isAvailable = GetSkillDetail(index1, index2);
		if(skillName ~= nil) then
			command.commandType = "skill";
			command.commandName = skillName;
			command.icon = icon;
		end
	elseif(cursorType == Kitty.DragAndDropManager.ACTION) then
		--------------------------------------------------------------------------------
		-- アクション
		PickupAction(index1);	-- カーソルクリア

		-- ショートカット
		local icon, macroName, count, locked, wore, continued = GetActionInfo(index1);
		local name = Kitty.SlotManager.getActionName(KittyComboGameTooltip, index1);
		if(name ~= nil) then
			name = Kitty.SlotManager.getPureName(name);	-- +70などの強化値を除去

			-- ショートカット判定
			local actionType = Kitty.CommandConfigFrame.getActionType(name);
			if(actionType == Kitty.DragAndDropManager.SKILL) then
				-- Skill
				command.commandType = "skill";
				command.commandName = name;
				command.icon = icon;
			elseif(actionType == Kitty.DragAndDropManager.SUIT_SKILL) then
				-- SuitSkill
				command.commandType = "suitskill";
				command.commandName = name;
				command.icon = icon;
			elseif(actionType == Kitty.DragAndDropManager.EMOTE) then
				-- Emote
				command.commandType = "emote";
				command.commandName = name;
				command.icon = icon;
			elseif(actionType == Kitty.DragAndDropManager.MACRO) then
				-- Macro
				command.commandType = "macro";
				command.commandName = name;
				command.icon = icon;
			else	-- actionType == Kitty.DragAndDropManager.ACTION
				-- Action
				command.commandType = "action";
				command.commandName = name;
				command.icon = icon;
			end
		end
	elseif(cursorType == Kitty.DragAndDropManager.BAG) then
		--------------------------------------------------------------------------------
		-- アイテム
		PickupBagItem(index1);	-- カーソルクリア

		SendWarningMsg(Kitty.T("message", "CanNotUseItemMessage"));
		SendWarningMsg(Kitty.T("message", "CanUseFromShortcutMessage"));
	elseif(cursorType == Kitty.DragAndDropManager.EQUIP) then
		--------------------------------------------------------------------------------
		-- 装備
		PickupEquipmentItem(index1);	-- カーソルクリア

		SendWarningMsg(Kitty.T("message", "CanNotSetEquipmentMessage"));
	elseif(cursorType == Kitty.DragAndDropManager.BANK) then
		--------------------------------------------------------------------------------
		-- 銀行
		PickupBankItem(index1);	-- カーソルクリア

		SendWarningMsg(Kitty.T("message", "CanNotSetBankItemMessage"));
	elseif(cursorType == Kitty.DragAndDropManager.EMOTE) then
		--------------------------------------------------------------------------------
		-- エモート
		ItemClipboard_Clear();	-- カーソルクリア

		local index, name1, name2, icon = GetEmoteInfo(index1 - 1);
		if(index ~= nil) then
			-- コマンド取得
			command.commandType = "emote";
			command.commandName = name1;
			command.value = index;
			command.icon = icon;
		end
	elseif(cursorType == Kitty.DragAndDropManager.MACRO) then
		--------------------------------------------------------------------------------
		-- マクロ
		ItemClipboard_Clear();	-- カーソルクリア

		local index, name, body = GetMacroInfo(index1);
		if(index ~= nil) then
			local icon = GetMacroIconInfo(index);
			command.commandType = "macro";
			command.commandName = name;
			command.icon = icon;
		end
	elseif(cursorType == Kitty.DragAndDropManager.SUIT_SKILL) then
		--------------------------------------------------------------------------------
		-- スーツスキル(リスト)
		ItemClipboard_Clear();	-- カーソルクリア

		local name, icon = skillManager:getSuitSkill(index1);
		if(name ~= nil) then
			command.commandType = "suitskill";
			command.commandName = name;
			command.icon = icon;
		end
	elseif(cursorType == Kitty.DragAndDropManager.SUIT_SKILL_PLATE) then
		--------------------------------------------------------------------------------
		-- スーツスキル(プレート)
		ItemClipboard_Clear();	-- カーソルクリア

		local name, icon = SkillPlateUpdate(index1);
		if(name ~= nil) then
			command.commandType = "suitskill";
			command.commandName = name;
			command.icon = icon;
		end
	else
		-- Error
		Kitty.sendMessage("Error: Not support. CursorType:[" .. cursorType .. "]", "DEFAULT", 1, 0, 0);
	end

	-- 明細更新
	Kitty.CommandConfigFrame.onRedraw(KittyCommandConfigFrame, tonumber(KittyCommandConfigFrame_ScrollBar:GetValue()), 10);

	skillManager = nil;
end;

-- リストの明細行に表示されている項目を内部データへ格納
Kitty.CommandConfigFrame.allowUpdateData = true;
Kitty.CommandConfigFrame.updateData = function(this)
	if(Kitty.CommandConfigFrame.allowUpdateData) then
		Kitty.debugPrint("Kitty.CommandConfigFrame.updateData");
	else
		Kitty.debugPrint("Kitty.CommandConfigFrame.updateData Canceled.");
		return;
	end

	local info = {};
	-- ヘッダ部の更新
	local commandListNameEditBox = getglobal("KittyCommandConfigFrame_CommandListNameEditBox");
	assert(commandListNameEditBox, "KittyCommandConfigFrame_CommandListNameEditBox is nil");
	info.name = commandListNameEditBox:GetText();

	local buildDropDownMenu = getglobal("KittyCommandConfigFrame_BuildDropDownMenu");
	assert(buildDropDownMenu, "KittyCommandConfigFrame_BuildDropDownMenu is nil");
	info.class = UIDropDownMenu_GetText(buildDropDownMenu);

	local targetDropDownMenu = getglobal("KittyCommandConfigFrame_TargetDropDownMenu");
	assert(targetDropDownMenu, "KittyCommandConfigFrame_TargetDropDownMenu is nil");
	info.target = UIDropDownMenu_GetText(targetDropDownMenu);

	Kitty.CommandConfigFrame.targetCommandList.info = info;

	if(this ~= nil) then
		-- リスト内部の行の更新
		local index = KittyCommandConfigFrame_ScrollBar:GetValue() + this:GetID() - 1;
		local command = {};
		for index, condition in ipairs(Kitty.CommandConfigFrame.targetCommandList[index]) do
			command[index] = condition;
		end

		-- 有効/無効チェックボタン
		local checkButton = getglobal(this:GetName() .. "CheckButton");
		assert(checkButton, this:GetName() .. "CheckButton is nil");
		command.enable = checkButton:IsChecked();

		-- ドロップダウンメニュー(コマンドタイプ)
		local dropDownMenu = getglobal(this:GetName() .. "DropDownMenu");
		assert(dropDownMenu, this:GetName() .. "DropDownMenu is nil");
		command.commandType = UIDropDownMenu_GetText(dropDownMenu);

		if(command.commandType ~= "proc") then
			-- エディットボックス(コマンド名)
			local commandNameEditBox = getglobal(this:GetName() .. "CommandNameEditBox");
			assert(commandNameEditBox, this:GetName() .. "CommandNameEditBox is nil");
			command.commandName = commandNameEditBox:GetText();
		else
			-- ドロップダウンメニュー(proc名)
			local commandDropDownMenu = getglobal(this:GetName() .. "CommandDropDownMenu");
			assert(dropDownMenu, this:GetName() .. "CommandDropDownMenu is nil");
			command.commandName = UIDropDownMenu_GetText(commandDropDownMenu);
		end

		-- コマンドアイコン
		if(command.commandType == "extra" or command.commandType == "proc" or command.commandType == "break" or command.commandType == "debug") then
			command.icon = nil;
		else
			command.icon = Kitty.CommandConfigFrame.targetCommandList[index].icon;
		end

		-- コマンド発動条件ボタン
--		local conditionButton = getglobal(this:GetName() .. "ConditionButton");
--		assert(conditionButton, this:GetName() .. "ConditionButton is nil");

		Kitty.CommandConfigFrame.targetCommandList[index] = command;
	end
end;

-- コマンドリスト一覧の明細表示
Kitty.CommandConfigFrame.onRedraw = function(this, start, count)
	local position = KittyCommandConfigFrame_ScrollBar:GetValue();
	if(start == position) then
		-- 再描画
		Kitty.CommandConfigFrame.onShowList(KittyCommandConfigFrame_ScrollBar, start, count);
	else
		-- スクロールバー更新(その後再描画される)
		KittyCommandConfigFrame_ScrollBar:SetValue(start);
	end
end;
Kitty.CommandConfigFrame.onShowList = function(this, start, count)
	Kitty.CommandConfigFrame.allowUpdateData = false;
	Kitty.debugPrint("Kitty.CommandConfigFrame.onShowList");

	if(count ~= nil) then
		count = 10;
	end

	-- スライダーの範囲を確定
	local aboveRange = start - 1;	-- 上方へスクロール可能なコマ数
	if(aboveRange < 0) then
		aboveRange = 0;
	end
	local belowRange = 0;
	if(Kitty.CommandConfigFrame.targetCommandList ~= nil) then
		belowRange = #Kitty.CommandConfigFrame.targetCommandList - count - (start - 1);	-- 下方へスクロール可能なコマ数
	end
	if(belowRange < 0) then
		belowRange = 0;
	end
	local range = aboveRange + belowRange + 1;

	if(range <= 0) then
		-- スライダー不要
		this:SetMinMaxValues(1, 1);
	else
		this:SetMinMaxValues(1, range);
	end

	-- Debug
	local min, max = this:GetMinMaxValues();
	Kitty.debugPrint("Scroll Value:[" .. tostring(this:GetValue()) .. "], Range:[" .. min .. ", " .. max .. "], Above:[" .. aboveRange .. "], Below:[" .. belowRange .. "]");

	-- 明細を表示
	local position = 1;
	for index=start, start+count-1 do
		local detailButton = getglobal("KittyCommandConfigFrame_Button" .. tostring(position));
		assert(detailButton, "KittyCommandConfigFrame_Button" .. tostring(position) .. " is nil.");

		-- 明細更新
		if(Kitty.CommandConfigFrame.targetCommandList ~= nil) then
			local command = Kitty.CommandConfigFrame.targetCommandList[index];
			if(command ~= nil) then
				Kitty.CommandConfigFrame.showDetail(detailButton, command);
				detailButton:Show();
			else
				detailButton:Hide();
			end
		end

		position = position + 1;
	end

	Kitty.CommandConfigFrame.allowUpdateData = true;
end;

-- 内部データをリストの明細行へ表示
Kitty.CommandConfigFrame.showDetail = function(this, command)
	-- 有効/無効チェックボタン
	local checkButton = getglobal(this:GetName() .. "CheckButton");
	assert(checkButton, this:GetName() .. "CheckButton is nil");
	checkButton:SetChecked(command.enable);

	-- コマンドアイコン
	local iconButton = getglobal(this:GetName() .. "ActionButtonIcon");
	assert(iconButton, this:GetName() .. "ActionButtonIcon is nil");
	if(command.icon ~= nil) then
		iconButton:SetFile(command.icon);
	else
		iconButton:SetFile("");
	end

	-- ドロップダウンメニュー(コマンドタイプ)
	local dropDownMenu = getglobal(this:GetName() .. "DropDownMenu");
	assert(dropDownMenu, this:GetName() .. "DropDownMenu is nil");
	UIDropDownMenu_SetText(dropDownMenu, tostring(command.commandType));

	-- エディットボックス(コマンド名)
	local editBox = getglobal(this:GetName() .. "CommandNameEditBox");
	assert(editBox, this:GetName() .. "CommandNameEditBox is nil");
	-- ドロップダウンメニュー(コマンド名)
	local commandDropDownMenu = getglobal(this:GetName() .. "CommandDropDownMenu");
	assert(dropDownMenu, this:GetName() .. "CommandDropDownMenu is nil");
	if(command.commandType ~= "proc") then
		-- エディットボックスを表示
		editBox:SetText(tostring(command.commandName));
		UIDropDownMenu_SetText(commandDropDownMenu, tostring(""));

		editBox:Show();
		commandDropDownMenu:Hide();
	else
		-- ドロップダウンメニューを表示
		editBox:SetText(tostring(""));
		UIDropDownMenu_SetText(commandDropDownMenu, tostring(command.commandName));

		editBox:Hide();
		commandDropDownMenu:Show();
	end

	-- コマンド発動条件ボタン
	local conditionButton = getglobal(this:GetName() .. "ConditionButton");
	assert(conditionButton, this:GetName() .. "ConditionButton is nil");
	-- 分割後の個数を表示
	conditionButton:SetText(Kitty.T("message", "Condition") .. "(" .. #Kitty.CommandConditionFrame.separate(command) .. ")");
end;

-- 名前から、スキル、マクロなど種別を判定する
Kitty.CommandConfigFrame.getActionType = function(actionName)
	local skillManager = Kitty.SkillManager(Kitty.cooldownManager);
	skillManager:update();

	local tabIndex, skillIndex, index = skillManager:getIndex(actionName);
	if(tabIndex ~= nil) then
		if(skillManager:isSuitSkill(actionName)) then
			-- SuitSkill
			return Kitty.DragAndDropManager.SUIT_SKILL, index;
		else
			-- Skill
			return Kitty.DragAndDropManager.SKILL, tabIndex, skillIndex;
		end
	end

	-- Emote
	local emotionManager = Kitty.EmotionManager();
	local emotionIndex = emotionManager:getIndex(actionName);
	if(emotionIndex ~= nil) then
		return Kitty.DragAndDropManager.EMOTE, emotionIndex;
	end

	-- Macro
	local macroIndex = Kitty.macroManager:getIndex(actionName);
	if(macroIndex ~= nil) then
		return Kitty.DragAndDropManager.MACRO, macroIndex;
	end

	-- Action
	local slotIndex = Kitty.slotManager:getIndex(actionName);
	if(slotIndex ~= nil) then
		return Kitty.DragAndDropManager.ACTION, slotIndex;
	end

	return nil;

	-- 以下は、未対応
	-- Bag: Kitty.DragAndDropManager.BAG
	-- Equip: Kitty.DragAndDropManager.EQUIP
	-- Bank: Kitty.DragAndDropManager.BANK
end;
