-- (C) 2012-2013 Miranda
--------------------------------------------------------------------------------
-- 名前空間作成
if(_G and not _G.Kitty) then
	_G.Kitty = {};
end

--------------------------------------------------------------------------------
Kitty.CommandListFrame = {};
-- 編集対象のコマンドリスト群
Kitty.CommandListFrame.allCommandList = nil;

-- UIコントロール/データ対応表
Kitty.CommandListFrame.relativeList = {
	-- 静的メッセージ一覧
	["KittyCommandListFrame_VersionLabel"] = "KittyCombo Version:[" .. Kitty.VERSION .. "]",
	["KittyCommandListFrame_TitleLabel"] = Kitty.T("message", "AllCommandListTitle"),

	["KittyCommandListFrame_CommandListNameLabel"] = Kitty.T("message", "CommandListNameLabel"),
	["KittyCommandListFrame_CommandListClassLabel"] = Kitty.T("message", "CommandListClassLabel"),
	["KittyCommandListFrame_CommandListTargetLabel"] = Kitty.T("message", "CommandListTargetLabel"),

	["KittyCommandListFrame_CreateButton"] = Kitty.T("message", "CreateButton"),
	["KittyCommandListFrame_InitializeButton"] = Kitty.T("message", "InitializeButton"),
	["KittyCommandListFrame_CloseButton"] = Kitty.T("message", "SaveAndCloseButton"),
	["KittyCommandListFrame_CancelButton"] = Kitty.T("message", "CancelButton"),
};

--------------------------------------------------------------------------------
-- 画面表示
Kitty.CommandListFrame.open = function()
	Kitty.CommandListFrame.allCommandList = Kitty.ListManager(Kitty.copyTable(Kitty.SettingManager.setting.command));

	Kitty.settingToComponent(nil, Kitty.CommandListFrame.relativeList);

	-- 明細表示
	Kitty.CommandListFrame.onRedraw(this, 1, 10);

	KittyCommandListFrame:Show();
end;
Kitty.CommandListFrame.update = function(allCommandList)
	Kitty.CommandListFrame.allCommandList = allCommandList;

	-- 明細表示
	Kitty.CommandListFrame.onRedraw(this, 1, 10);

	KittyCommandListFrame:Show();
end;

Kitty.CommandListFrame.save = function()
	Kitty.SettingManager.setting.command = nil;	-- GC
	Kitty.SettingManager.setting.command = Kitty.CommandListFrame.allCommandList;
end;
Kitty.CommandListFrame.close = function()
	Kitty.CommandListFrame.allCommandList = nil;

	KittyCommandListFrame:Hide();
end;

-- クリック時の処理
Kitty.CommandListFrame.onClick = function(this, isNew)
	KittyCommandListFrame:Hide();

	if(isNew == true) then
		Kitty.CommandConfigFrame.open(Kitty.CommandListFrame.allCommandList, 0, Kitty.CommandListFrame.update);
	else
		local index = KittyCommandListFrame_ScrollBar:GetValue() + this:GetParent():GetID() - 1;
		-- 参照・編集するコマンドリスト
		Kitty.CommandConfigFrame.open(Kitty.CommandListFrame.allCommandList, index, Kitty.CommandListFrame.update);
	end
end;

-- コマンドリスト一覧の明細表示
Kitty.CommandListFrame.onRedraw = function(this, start, count)
	local position = KittyCommandListFrame_ScrollBar:GetValue();
	if(start == position) then
		-- 再描画
		Kitty.CommandListFrame.onShowList(KittyCommandListFrame_ScrollBar, start, count);
	else
		-- スクロールバー更新(その後再描画される)
		KittyCommandListFrame_ScrollBar:SetValue(start);
	end
end;
Kitty.CommandListFrame.onShowList = function(this, start, count)
	if(count ~= nil) then
		count = 10;
	end

	-- スライダーの範囲を確定
	local aboveRange = start - 1;	-- 上方へスクロール可能なコマ数
	if(aboveRange < 0) then
		aboveRange = 0;
	end
	local belowRange = 0;
	if(Kitty.CommandListFrame.allCommandList ~= nil) then
		belowRange = #Kitty.CommandListFrame.allCommandList - count - (start - 1);	-- 下方へスクロール可能なコマ数
	end
	if(belowRange < 0) then
		belowRange = 0;
	end
	local range = aboveRange + belowRange + 1;

	if(range <= 0) then
		-- スライダー不要
		this:SetMinMaxValues(1, 1);
	else
		this:SetMinMaxValues(1, range);
	end

	-- Debug
	local min, max = this:GetMinMaxValues();
	Kitty.debugPrint("Scroll Value:[" .. tostring(this:GetValue()) .. "], Range:[" .. min .. ", " .. max .. "], Above:[" .. aboveRange .. "], Below:[" .. belowRange .. "]");

	-- 明細を表示
	local position = 1;
	for index=start, start+count-1 do
		-- 明細
		local detailButton = getglobal("KittyCommandListFrame_Button" .. tostring(position));
		assert(detailButton, "KittyCommandListFrame_Button" .. tostring(position) .. " is nil.");
		-- 明細のコマンドリスト名
		local buttonName = getglobal(detailButton:GetName() .. "Name");
		assert(buttonName, detailButton:GetName() .. "Name is nil.");
		-- 明細の対象クラス
		local buttonClass = getglobal(detailButton:GetName() .. "Class");
		assert(buttonClass, detailButton:GetName() .. "Class is nil.");
		-- 明細のターゲット
		local buttonTarget = getglobal(detailButton:GetName() .. "Target");
		assert(buttonTarget, detailButton:GetName() .. "Target is nil.");
		-- 明細の編集ボタン
		local buttonEdit = getglobal(detailButton:GetName() .. "EditButton");
		assert(buttonEdit, detailButton:GetName() .. "EditButton is nil.");

		if(Kitty.CommandListFrame.allCommandList ~= nil) then
			local command = Kitty.CommandListFrame.allCommandList[index];
			if(command ~= nil) then
				local info = command.info;
				if(info ~= nil) then
					-- コマンドリスト名
					buttonName:SetText(tostring(info.name));
					if(Kitty.BuildManager.matchWithTarget("player", info.class)) then
						-- クラス(対象)
						buttonClass:SetColor(0.0, 1.0, 1.0);
						buttonClass:SetText(tostring(info.class));
					else
						-- クラス(非対象)
						buttonClass:SetColor(1.0, 1.0, 1.0);
						buttonClass:SetText(tostring(info.class));
					end
					-- ターゲット
					buttonTarget:SetText(tostring(info.target));
					-- ボタン名
					buttonEdit:SetText(Kitty.T("message", "EditButton"));
				else
					buttonName:SetText("None:[info]");
				end
				detailButton:Show();
			else
				buttonName:SetText("None:[command]");
				detailButton:Hide();
			end
		end

		position = position + 1;
	end

	-- コマンドリスト名重複チェック
	local duplicatedCommandNameList = {};
	if(Kitty.CommandListFrame.allCommandList ~= nil) then
		for index, commandList in ipairs(Kitty.CommandListFrame.allCommandList) do
			if(commandList.info ~= nil) then
				if(duplicatedCommandNameList[commandList.info.name] == nil) then
					duplicatedCommandNameList[commandList.info.name] = 1;
				else
					duplicatedCommandNameList[commandList.info.name] = duplicatedCommandNameList[commandList.info.name] + 1;
				end
			end
		end
	end

	position = 1;
	for index=start, start+count-1 do
		-- 明細
		local detailButton = getglobal("KittyCommandListFrame_Button" .. tostring(position));
		assert(detailButton, "KittyCommandListFrame_Button" .. tostring(position) .. " is nil.");
		-- 明細のコマンドリスト名
		local buttonName = getglobal(detailButton:GetName() .. "Name");
		assert(buttonName, detailButton:GetName() .. "Name is nil.");

		if(Kitty.CommandListFrame.allCommandList ~= nil) then
			local command = Kitty.CommandListFrame.allCommandList[index];
			if(command ~= nil) then
				local info = command.info;
				if(info ~= nil) then
					if(duplicatedCommandNameList[info.name] > 1) then
						buttonName:SetColor(1.0, 0.0, 0.0);
					else
						buttonName:SetColor(1.0, 1.0, 1.0);
					end
				end
			end
		end
		position = position + 1;
	end
end;
