local Kitty = {};
_G.Kitty = Kitty;

-- ロード
local result, errorMessage = pcall(dofile, [[lib/KittyLib.lua]]);
assert(result, errorMessage);

-- トレース
--Kitty.trace("cr", {"C", "trace", "dump", "print", "sendMessage", "error", "getStackTrace", "printStackTrace", "formatStackTrace"});

-- 実行部
print("--- Start ---");

--[[
UIDROPDOWNMENU_MENU_LEVEL = nil;
UIDROPDOWNMENU_MENU_VALUE = nil;
UIDropDownMenu = {};
UIDropDownMenu.width = 0;
UIDropDownMenu.text = "";
UIDropDownMenu.init = nil;
UIDropDownMenu.data = {{}, {}};

UIDropDownMenu_SetWidth = function(self, width)
	UIDropDownMenu.width = width;
end;
UIDropDownMenu_GetText = function(self)
	return UIDropDownMenu.text;
end;
UIDropDownMenu_SetText = function(self, text)
	UIDropDownMenu.text = text;
end;
UIDropDownMenu_Initialize = function(self, init)
	UIDropDownMenu.init = init;
end;
CloseDropDownMenus = function(level)
end;
UIDropDownMenu_AddButton = function(settingsList, level)
	table.insert(UIDropDownMenu.data[level], settingsList);
end;
UIDropDownMenu_Open = function(level, index)	-- for Test
	UIDROPDOWNMENU_MENU_LEVEL = level;
	if(level == 2) then
		UIDROPDOWNMENU_MENU_VALUE = UIDropDownMenu.data[level-1][index].value;
	else
		UIDROPDOWNMENU_MENU_VALUE = nil;
	end

	UIDropDownMenu.data[level] = {};
	UIDropDownMenu.init(nil);

	-- Dump
	UIDropDownMenu_Show(level);
end;
UIDropDownMenu_Select = function(level, index)	-- for Test
	local setting = UIDropDownMenu.data[level][index];
	if(not setting) then
		Kitty.print("setting is nil. Level:[" .. tostring(level) .. "], Index:[" .. tostring(index) .. "]");
	end
	if(setting and setting.func) then
		setting.func(Kitty.Button("ID", "Button", setting.text, setting.value));
	end
end;
UIDropDownMenu_Show = function(level)	-- for Test
	for index, list in ipairs(UIDropDownMenu.data[level]) do
		Kitty.print(Kitty.DropDown.SettingtoString(list));
	end
end;

-- クラス
Kitty.Button = {};
-- インスタンスメソッド
Kitty.Button.GetID = function(self)
	return self.id;
end;
Kitty.Button.GetName = function(self)
	return self.name;
end;
Kitty.Button.GetText = function(self)
	return self.text;
end;
-- 文字列表現
Kitty.Button.toString = function(self)
	return "ID:[" .. tostring(self.id) .. "], Name:[" .. tostring(self.name) .. "], Value:[" .. tostring(self.value) .. "], Text:[" .. tostring(self.text) .. "]";
end;
-- コンストラクタ
Kitty.Button.constructor = function(self, id, name, text, value)
	-- データメンバ
	local obj = {};
	obj.id = id;
	obj.name = name;
	obj.text = text;
	obj.value = value;

	-- 関数の付与、文字列表現サポート
	return setmetatable(obj, {__index=Kitty.Button, __tostring=Kitty.Button.toString});
end;
setmetatable(Kitty.Button, {__call=Kitty.Button.constructor});	-- コンストラクタを指定
--]]

--[[
-- 翻訳用書式化関数
Kitty.format = function(text, ...)
	if(text ~= nil and text ~= "") then
		for index=1, select("#", ...) do
			text = text:gsub("<<" .. index .. ">>", (select(index, ...)));	-- selectの戻り値の個数をひとつにする
		end
	end
	return tostring(text);
end

Kitty.print(Kitty.format("AA[<<2>>]BB[<<1>>]CC", "111", "222", "333"));
--]]

--[[
do
	Kitty.print(string.toArray("asa{}"));
end
--]]

do
--[[
	local s, e, eventCode = string.find("iD : 【test】", "[Ii][Dd]%s-:%s-【(.-)】");
	Kitty.print(s, e, eventCode);
--]]
end

do
--[[
	local orgFunc = function(str)
		print("[" .. tostring(str) .. "]");
	end;
	-- Hookのテスト
	local overrideFunc = function(orgFunc, ...)
		print(...);
		orgFunc(...);
	end;

	orgFunc = Kitty.HookManager.override(orgFunc, overrideFunc);
--	Kitty.print(orgFunc);
	orgFunc(1, 2, 3);
--]]
end

do
--[[
	-- イテレータのテスト
	Kitty.iterator = function()
		local memberList = {"player", "party1", "party2", "party3", "party4", "party5"};

		-- イテレータ
		local iterator = function(_, index)
			for _, unit in pairs(memberList) do
				local name = "Hello";
	--			local name = nil;
				if(name ~= nil) then
					index = index + 1;
					coroutine.yield(index, unit, name);
				end
			end
		end

		-- iterator関数、状態オブジェクト、インデックス初期値
		return coroutine.wrap(iterator), nil, 0;
	end;

	for index, unit, name in Kitty.iterator() do
		Kitty.print(index, unit, name);
	end
--]]
end

do
--[[
	-- コルーティンのテスト
	local co = coroutine.wrap(
		function()
			coroutine.yield("aaa", 1, 2);
			coroutine.yield("bbb", 1, 2);
			coroutine.yield("ccc", 1, 2);
			coroutine.yield("ddd", 1, 2);
		end
	);

	for a, b, c in co do
		Kitty.print(a, b, c);
	end
--]]
end

do
--[[
	-- Hook
	local orgFunc = function(str)
		print("[" .. str .. "]");
	end;
	local preFunc = function(...)
		print("PreFunc.");
	end;
	local postFunc = function(...)
		print("PostFunc.");
	end;
	local countFunc = function(index)
		return function()
			print(index);
		end
	end;

	orgFunc = Kitty.HookManager("Test", orgFunc, countFunc(1), countFunc(2));
	orgFunc = Kitty.HookManager("Test", orgFunc, countFunc(11), countFunc(12));
	orgFunc = Kitty.HookManager("Test", orgFunc, countFunc(21), countFunc(22));
	orgFunc = Kitty.HookManager("Test", orgFunc, countFunc(31), countFunc(32));
	orgFunc = Kitty.HookManager("Test", orgFunc, countFunc(41), countFunc(42));

	orgFunc("@@@@");
	Kitty.print(orgFunc);

	orgFunc = orgFunc:getOriginalFunction();
	orgFunc("@@@@");
	Kitty.print(orgFunc);
--]]
end

Kitty.protectFunction(function()
--	Kitty.print("Hello World!");
--	orgFunc = Kitty.protectFunction(orgFunc);
end)();

print("--- E n d ---");
