----------------------------------------------------------------------------------------------------------
-- Lootomatic
--
-- written by PetraAreon
-- released under the Creative Commons License By-Nc-Sa: http://creativecommons.org/licenses/by-nc-sa/3.0/
--
-- Get the current version at:   http://rom.curse.com/downloads/rom-addons/details/lootomatic.aspx
----------------------------------------------------------------------------------------------------------

function Lootomatic_OnLoad(this)
	local Sol = LibStub("Sol");

	local BootyCloseTimer = nil;
	Lootomatic = {
		ListSelect  = nil,
		Name        = "Lootomatic",
		Version     = "v0.4",
		Loaded      = false;
		MissingLang = nil;
		Timer       = 0,
		Fader       = 0,
		Defaults    =
			{
			AM              = true,	-- Enabled in AddonManager?
			Enabled         = true,	-- Lootomatic enabled?
			Interval        = 1,
			Roll            = 3,
			AutoLoot        = 3,
			AutoPass        = 1,
			AutoGreed       = 1,
			AutoLootBoss    = false,
			MoveBooty       = false,
			CloseBooty      = false,
			ChatOutput      = false,
			LongTooltips    = true,
			IgnoreQuestItem = true,
			Chatfilter      = {true,true,true,true,true,true,},
			RollFrameX      = 0,
			RollFrameY      = -60,
			RollFrameD      = -64,
			ItemFilterX     = 450,
			ItemFilterY     = 300,
			ItemFilter      = {},
			},
		};

	Lootomatic.Language = string.upper(string.sub(GetLanguage(),1,2));

	local func, err = loadfile("Interface/Addons/Lootomatic/Locales/"..Lootomatic.Language..".lua");
	if (err) then
		dofile("Interface/Addons/Lootomatic/Locales/EN.lua");
		Lootomatic.MissingLang = Lootomatic.Language;
		Lootomatic.Language    = "EN";
	else
		dofile("Interface/Addons/Lootomatic/Locales/"..Lootomatic.Language..".lua");
	end;

	Lootomatic.QualityColors =
		{
		"ffffff",
		"00ff00",
		"0072bc",
		"c805f8",
		"f68e56",
		"a37d50",
		};

	Lootomatic.ListEntries =
		{
		"|cffaaaaaa"                       ..Lootomatic.Strings.Never.."|r",
		"|cff"..Lootomatic.QualityColors[1]..Lootomatic.Strings.White.."|r",
		"|cff"..Lootomatic.QualityColors[2]..ITEM_QUALITY1_DESC.."|r",
		"|cff"..Lootomatic.QualityColors[3]..ITEM_QUALITY2_DESC.."|r",
		"|cff"..Lootomatic.QualityColors[4]..ITEM_QUALITY3_DESC.."|r",
		"|cff"..Lootomatic.QualityColors[5]..ITEM_QUALITY4_DESC.."|r",
		"|cff"..Lootomatic.QualityColors[6]..ITEM_QUALITY5_DESC.."|r",
		[101] = "|cffff0000"..LOOT_FREE_FOR_ALL.."|r",
		};

 	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("BOOTY_SHOW");
	this:RegisterEvent("CHAT_MSG_SYSTEM_GET");

	SLASH_LOOTOMATIC1 = "/lootomatic";
	SlashCmdList["LOOTOMATIC"] = Lootomatic_ItemFilter_Toggle;
end;

function Lootomatic_OnEvent(event)
	    if (event == "VARIABLES_LOADED") then
		if (Lootomatic_Settings == nil) then
			Lootomatic_Settings = Lootomatic.Defaults;
		else
			if (Lootomatic.Version ~= Lootomatic_Settings.Version) then
				DEFAULT_CHAT_FRAME:AddMessage("|cffFFFE80["..Lootomatic.Name.."]:|r "..string.gsub(Lootomatic.Strings.Updated,"#VERSION#",Lootomatic.Version),0.7,0.7,0.7);

				local Transfer_ItemFilter = Lootomatic_Settings.ItemFilter;

				Lootomatic_Settings = Lootomatic.Defaults;

				Lootomatic_Settings.ItemFilter = Transfer_ItemFilter;
			end;
		end;
		Lootomatic_Settings.Version = Lootomatic.Version;

		if (AddonManager) then
			local addon = {
				name = "Lootomatic",
				version = Lootomatic.Version,
				author = "PetraAreon",
				description = Lootomatic.Strings.Description,
				icon = "Interface/Addons/Lootomatic/Textures/minimap-normal.png",
				category = "Inventory",
				configFrame = Lootomatic_SettingsFrame,
				slashCommands = SLASH_LOOTOMATIC1,
				disableScript = Lootomatic_Disable,
				enableScript = Lootomatic_Enable,
			};
			if (AddonManager.RegisterAddonTable) then
				AddonManager.RegisterAddonTable(addon);
			else
				AddonManager.RegisterAddon(addon.name, addon.description, addon.icon, addon.category, addon.configFrame, addon.slashCommands, addon.miniButton, addon.version, addon.author);
			end;
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cffFFFE80["..Lootomatic.Name.." "..Lootomatic.Version.."]:|r "..Lootomatic.Strings.Loaded,0.7,0.7,0.7);
		end;

		if (Lootomatic.MissingLang ~= nil) then
			DEFAULT_CHAT_FRAME:AddMessage("|cffFFFE80["..Lootomatic.Name.." "..Lootomatic.Version.."]:|r No locales found for your language \""..Lootomatic.MissingLang.."\", switching to fallback \"EN\". If you want your language to be added, please contact the author PetraAreon ;-)",0.7,0.7,0.7);
		end;

		Lootomatic_ItemFilterFrame:SetSize(Lootomatic_Settings.ItemFilterX,Lootomatic_Settings.ItemFilterY);
		Lootomatic_ItemFilter_MaxSize();
		Lootomatic_ItemFilter_ResizeElements();

		Lootomatic_ItemFilterFrame_Header        :SetText(Lootomatic.Name.." - "..Lootomatic.Strings.ItemFilter);
		Lootomatic_ItemFilterFrame_Itemname_Title:SetText(Lootomatic.Strings.Itemname);
		Lootomatic_ItemFilterFrame_Looting_Title :SetText(Lootomatic.Strings.Looting);
		Lootomatic_ItemFilterFrame_Rolling_Title :SetText(Lootomatic.Strings.Rolling);

		Lootomatic_SettingsFrame_Header                     :SetText(Lootomatic.Name.." - "..Lootomatic.Strings.Settings);
		Lootomatic_SettingsFrame_BootyFrame_Header_Text     :SetText(Lootomatic.Strings.Settings_BootyHeader);
		Lootomatic_SettingsFrame_BootyFrame_Move_Text       :SetText(Lootomatic.Strings.Settings_BootyMove);
		Lootomatic_SettingsFrame_BootyFrame_Close_Text      :SetText(Lootomatic.Strings.Settings_BootyClose);
		Lootomatic_SettingsFrame_Chatframe_Header_Text      :SetText(Lootomatic.Strings.Settings_ChatHeader);
		Lootomatic_SettingsFrame_Chatframe_ChatOutput_Text  :SetText(Lootomatic.Strings.Settings_ChatOutput);
		Lootomatic_SettingsFrame_Rollframe_Header_Text      :SetText(Lootomatic.Strings.Settings_RollHeader);
		Lootomatic_SettingsFrame_Rollframe_LongTooltips_Text:SetText(Lootomatic.Strings.Settings_RollTooltips);
		Lootomatic_SettingsFrame_Rollframe_IgnoreQuest_Text :SetText(Lootomatic.Strings.Settings_RollIgnoreQuest);
		Lootomatic_SettingsFrame_Rollframe_IgnoreQuest      :Disable();
		Lootomatic_SettingsFrame_Rollframe_Position         :SetText(Lootomatic.Strings.Settings_RollPosition);
		Lootomatic_SettingsFrame_Rollframe_PositionSave     :SetText(Lootomatic.Strings.Settings_RollPositionSave);
		Lootomatic_SettingsFrame_Rollframe_PositionCancel   :SetText(Lootomatic.Strings.Settings_RollPositionCancel);
		Lootomatic_SettingsFrame_Rollframe_Slider           :SetValue(Lootomatic_Settings.Interval*10);

		Lootomatic_RollFrameGhosts_1       :ClearAllAnchors();
		Lootomatic_RollFrameGhosts_1       :SetAnchor("BOTTOM", "BOTTOM", UIParent, Lootomatic_Settings.RollFrameX, Lootomatic_Settings.RollFrameY);
		Lootomatic_RollFrameGhosts_1_Okay  :SetText(Lootomatic.Strings.Settings_RollPositionOkay);
		Lootomatic_RollFrameGhosts_1_Reset :SetText(Lootomatic.Strings.Settings_RollPositionReset);
		Lootomatic_RollFrameGhosts_1_Slider:SetValue(Lootomatic_Settings.RollFrameD);
		Lootomatic_RollFrameGhosts_Move();

		Lootomatic_RollFrameGhosts_2:SetAlpha(0.4);
		Lootomatic_RollFrameGhosts_3:SetAlpha(0.4);
		Lootomatic_RollFrameGhosts_4:SetAlpha(0.4);
		Lootomatic_RollFrameGhosts_5:SetAlpha(0.4);
		Lootomatic_RollFrameGhosts_6:SetAlpha(0.4);

		GroupLootFrame1:ClearAllAnchors(); GroupLootFrame1:SetAnchor("CENTER","CENTER", Lootomatic_RollFrameGhosts_1, 0, 0);
		GroupLootFrame2:ClearAllAnchors(); GroupLootFrame2:SetAnchor("CENTER","CENTER", Lootomatic_RollFrameGhosts_2, 0, 0);
		GroupLootFrame3:ClearAllAnchors(); GroupLootFrame3:SetAnchor("CENTER","CENTER", Lootomatic_RollFrameGhosts_3, 0, 0);
		GroupLootFrame4:ClearAllAnchors(); GroupLootFrame4:SetAnchor("CENTER","CENTER", Lootomatic_RollFrameGhosts_4, 0, 0);
		GroupLootFrame5:ClearAllAnchors(); GroupLootFrame5:SetAnchor("CENTER","CENTER", Lootomatic_RollFrameGhosts_5, 0, 0);
		GroupLootFrame6:ClearAllAnchors(); GroupLootFrame6:SetAnchor("CENTER","CENTER", Lootomatic_RollFrameGhosts_6, 0, 0);

	 	UIDropDownMenu_Initialize   (Lootomatic_Dropdown, Lootomatic_MainMenue);
		UIDropDownMenu_Initialize   (Lootomatic_ItemFilterFrame_Looting, Lootomatic_ItemFilter_LootmenuItems);
		UIDropDownMenu_SetSelectedID(Lootomatic_ItemFilterFrame_Looting, 1);
		UIDropDownMenu_Initialize   (Lootomatic_ItemFilterFrame_Rolling, Lootomatic_ItemFilter_RollmenuItems);
		UIDropDownMenu_SetSelectedID(Lootomatic_ItemFilterFrame_Rolling, 1);

		------------------------------------------------------------------------------------------------------------------------------------
		--                       !WARNING! Modifying Original Content                                                                     --
		------------------------------------------------------------------------------------------------------------------------------------
		Sol.hooks.Hook("Lootomatic", "ChatFrame_OnEvent",     Lootomatic_ChatFrame_OnEvent);                                              -- Filter Chatframe
		Sol.hooks.Hook("Lootomatic", "BagItemButton_OnClick", Lootomatic_BagItemButton_OnClick);                                          -- Add to ItemFilter by Shift-Clicking Items in Bag
		Sol.hooks.Hook("Lootomatic", "RollOnLoot",            Lootomatic_RollOnLoot);                                                     -- Add to ItemFilter by Ctrl-Clicking in Rollframe
		Sol.hooks.Hook("Lootomatic", "Hyperlink_Assign",      Lootomatic_Hyperlink_Assign);                                               -- Add Lootomatic-Itemfilter line to item-rightclick in chatframe
		LootomaticBackup_GROUPLOOT_ROLL   = GROUPLOOT_ROLL;                                                                               --
		LootomaticBackup_GROUPLOOT_GREED  = GROUPLOOT_GREED;                                                                              -- Storing original Tooltiptext for further processing
		LootomaticBackup_GROUPLOOT_CANCEL = GROUPLOOT_CANCEL;                                                                             --
		Lootomatic_RollOnLoot_Tooltips();                                                                                                 -- Adding Lootomatic Textlines to the Tooltip of RollFrames
		------------------------------------------------------------------------------------------------------------------------------------

	 	SaveVariables("Lootomatic_Settings");
		Lootomatic.Loaded = true;

	elseif (Lootomatic.Loaded == true and event == "BOOTY_SHOW") then
		if (Lootomatic_Settings.Enabled == true and (not AddonManager or Lootomatic_Settings.AM == true)) then
			if (Lootomatic_Settings.MoveBooty==true) then
				local x,y = GetCursorPos();
				BootyFrame:ClearAllAnchors();
				BootyFrame:SetAnchor("TOPLEFT", "TOPLEFT", UIParent, x/GetUIScale()-40, y/GetUIScale()-56);
			end;
			Lootomatic_CheckBootyFrame();
		end;
	elseif (Lootomatic.Loaded == true and event == "CHAT_MSG_SYSTEM_GET") then
 		if (Lootomatic_Settings.Enabled == true and (not AddonManager or Lootomatic_Settings.AM == true)) then
 			if (Lootomatic_Settings.ChatOutput == true) then
				DEFAULT_CHAT_FRAME:AddMessage("|cffFFFE80["..Lootomatic.Name.."]:|r "..arg1,0.7,0.7,0.7);
 			end;
 			Lootomatic_DropGarbage();
 		end;
	end;
end;

function Lootomatic_OnUpdate(elapsedTime)
	if (Lootomatic.Loaded == true) then
		if (Lootomatic_Settings.Enabled == true and (not AddonManager or Lootomatic_Settings.AM == true)) then
			if (Lootomatic.Fader>0) then
				Lootomatic.Fader = Lootomatic.Fader-elapsedTime;
				if (Lootomatic.Fader<0) then Lootomatic.Fader=0; end;
				Lootomatic_Credits:SetAlpha(1-Lootomatic.Fader);
			elseif (Lootomatic.Fader<0) then
				Lootomatic.Fader = Lootomatic.Fader+elapsedTime;
				if (Lootomatic.Fader>0) then Lootomatic.Fader=0; Lootomatic_Credits:Hide(); end;
				Lootomatic_Credits:SetAlpha(0-Lootomatic.Fader);
			end;

			if (BootyCloseTimer ~= nil) then
				if (GetTickCount()-BootyCloseTimer>500) then BootyFrame:Hide(); BootyCloseTimer=nil; end;
			end;

			Lootomatic.Timer = Lootomatic.Timer + elapsedTime;
			if (Lootomatic.Timer >= Lootomatic_Settings.Interval) then
				Lootomatic.Timer = 0;

				if (Lootomatic_Settings.Roll ~= GetLootThreshold()+2) then
					Lootomatic_Settings.Roll = GetLootThreshold()+2;
					DEFAULT_CHAT_FRAME:AddMessage("|cffFFFE80["..Lootomatic.Name.."]:|r "..string.gsub(Lootomatic.Strings.RollChange,"#ROLLSETTING#","["..Lootomatic.ListEntries[Lootomatic_Settings.Roll].."]"),0.7,0.7,0.7);
				end;

				if (GroupLootFrame1:IsVisible() == true) then Lootomatic_CheckRollFrame(GroupLootFrame1.lootIndex); end;
				if (GroupLootFrame2:IsVisible() == true) then Lootomatic_CheckRollFrame(GroupLootFrame2.lootIndex); end;
				if (GroupLootFrame3:IsVisible() == true) then Lootomatic_CheckRollFrame(GroupLootFrame3.lootIndex); end;
				if (GroupLootFrame4:IsVisible() == true) then Lootomatic_CheckRollFrame(GroupLootFrame4.lootIndex); end;
				if (GroupLootFrame5:IsVisible() == true) then Lootomatic_CheckRollFrame(GroupLootFrame5.lootIndex); end;
				if (GroupLootFrame6:IsVisible() == true) then Lootomatic_CheckRollFrame(GroupLootFrame6.lootIndex); end;
			end;
		end;
	end;
end;


function Lootomatic_Sort_SearchResults(a,b) -- Sort the Searchresults of the ItemFilter, to find out the longest and therefore most precise match
	if (string.len(string.gsub(a[1],"\*","")) > string.len(string.gsub(b[1],"\*",""))) then return true;
	else return false; end;
end;

function Lootomatic_CheckRollFrame(Rollframe)
	if (Rollframe ~= nil) then
		name, icon, itemCount, quality = GetLootRollItemInfo(Rollframe);
		if (name ~= nil and quality ~= nil) then
			local lootmode = nil;
			if (quality <= Lootomatic_Settings.AutoGreed - 2) then lootmode = "greed"; end;
			if (quality <= Lootomatic_Settings.AutoPass  - 2) then lootmode = "pass";  end;

			local SearchResults = {};
			for i=1,table.maxn(Lootomatic_Settings.ItemFilter) do
				if (string.match(name, "^"..string.gsub(string.gsub(Lootomatic_Settings.ItemFilter[i]["Name"], "(%W)", "%%%1"),"%%%*",".*").."$") ~= nil
				and Lootomatic_Settings.ItemFilter[i]["Char"] == UnitName("player") ) then
					table.insert(SearchResults,{Lootomatic_Settings.ItemFilter[i]["Name"],Lootomatic_Settings.ItemFilter[i]["Roll"]});
				end;
			end;
			if (table.maxn(SearchResults)==0) then
				for i=1,table.maxn(Lootomatic_Settings.ItemFilter) do
					if (string.match(name, "^"..string.gsub(string.gsub(Lootomatic_Settings.ItemFilter[i]["Name"], "(%W)", "%%%1"),"%%%*",".*").."$") ~= nil
					and Lootomatic_Settings.ItemFilter[i]["Char"] == nil ) then
						table.insert(SearchResults,{Lootomatic_Settings.ItemFilter[i]["Name"],Lootomatic_Settings.ItemFilter[i]["Roll"]});
					end;
				end;
			end;
			if (table.maxn(SearchResults)>0) then
				table.sort(SearchResults,Lootomatic_Sort_SearchResults);
				    if (SearchResults[1][2]==2) then lootmode="pass";
				elseif (SearchResults[1][2]==3) then lootmode="greed";
				elseif (SearchResults[1][2]==4) then lootmode="roll";
				elseif (SearchResults[1][2]==5) then lootmode=nil; end;
			end;

			if (lootmode ~= nil) then
				Sol.hooks.GetOriginalFn("Lootomatic", "RollOnLoot") (Rollframe, lootmode);
			end;
		end;
	end;
end;

function Lootomatic_CheckBootyFrame()
	if (not IsShiftKeyDown()) then
		if (not(UnitSex("target")==3 and Lootomatic_Settings.AutoLootBoss==false)) then
			local closeframe = 0;
			local i=0;
			while true do
				i=i+1;
				texture, name, count, quality = GetBootyItemInfo(i);
				if (name ~= nil and quality ~= nil) then
					local loot = false;

					if (quality <= Lootomatic_Settings.AutoLoot - 2) then loot = true; else closeframe = 1; end;

					local SearchResults = {};
					for j=1,table.maxn(Lootomatic_Settings.ItemFilter) do
						if (string.match(name, "^"..string.gsub(string.gsub(Lootomatic_Settings.ItemFilter[j]["Name"], "(%W)", "%%%1"),"%%%*",".*").."$") ~= nil
						and Lootomatic_Settings.ItemFilter[j]["Char"] == UnitName("player") ) then
							table.insert(SearchResults,{Lootomatic_Settings.ItemFilter[j]["Name"],Lootomatic_Settings.ItemFilter[j]["Loot"]});
						end;
					end;
					if (table.maxn(SearchResults)==0) then
						for j=1,table.maxn(Lootomatic_Settings.ItemFilter) do
							if (string.match(name, "^"..string.gsub(string.gsub(Lootomatic_Settings.ItemFilter[j]["Name"], "(%W)", "%%%1"),"%%%*",".*").."$") ~= nil
							and Lootomatic_Settings.ItemFilter[j]["Char"] == nil ) then
								table.insert(SearchResults,{Lootomatic_Settings.ItemFilter[j]["Name"],Lootomatic_Settings.ItemFilter[j]["Loot"]});
							end;
						end;
					end;
					if (table.maxn(SearchResults)>0) then
						table.sort(SearchResults,Lootomatic_Sort_SearchResults);
						    if (SearchResults[1][2]==2) then loot = false; if (closeframe==0) then closeframe = 2; end;
						elseif (SearchResults[1][2]> 2) then loot = true; end;
					end;


					if (loot == true) then ClickBootyItem(i); end;
				else
					break;
				end;
 			if (closeframe == 2 and Lootomatic_Settings.CloseBooty == true) then BootyCloseTimer = GetTickCount(); end;
			end;
		end;
	end;
end;

function Lootomatic_DropGarbage()
	local garbagecount = 0;
	local bagitems, bagsize = GetBagCount();
	for i = 1,bagsize do
		local id, icon, name, count = GetBagItemInfo(i)
		if ( id and name ) then
			RemoveGarbageItem(id);

			local SearchResults = {};
			for j=1,table.maxn(Lootomatic_Settings.ItemFilter) do
				if (string.match(name, "^"..string.gsub(string.gsub(Lootomatic_Settings.ItemFilter[j]["Name"], "(%W)", "%%%1"),"%%%*",".*").."$") ~= nil
				and Lootomatic_Settings.ItemFilter[j]["Char"] == UnitName("player") ) then
					table.insert(SearchResults,{Lootomatic_Settings.ItemFilter[j]["Name"],Lootomatic_Settings.ItemFilter[j]["Loot"]});
				end;
			end;
			if (table.maxn(SearchResults)==0) then
				for j=1,table.maxn(Lootomatic_Settings.ItemFilter) do
					if (string.match(name, "^"..string.gsub(string.gsub(Lootomatic_Settings.ItemFilter[j]["Name"], "(%W)", "%%%1"),"%%%*",".*").."$") ~= nil
					and Lootomatic_Settings.ItemFilter[j]["Char"] == nil ) then
						table.insert(SearchResults,{Lootomatic_Settings.ItemFilter[j]["Name"],Lootomatic_Settings.ItemFilter[j]["Loot"]});
					end;
				end;
			end;
			if (table.maxn(SearchResults)>0) then
				table.sort(SearchResults,Lootomatic_Sort_SearchResults);
				if (SearchResults[1][2]==4) then
					InsertGarbageItem(id);
					garbagecount = garbagecount + 1;
				end;
			end;
		end;
	end;

	    if (garbagecount>3) then
		GarbageFrame:Show();
		StaticPopup_Show("DELETE_GARBAGE_ITEM");
 	elseif (garbagecount>0) then
 		DeleteGarbageItems();
 		GarbageFrame:Hide();
	end;
end;

-- function Lootomatic_CheckQuestItem(i)
-- 	link = GetBagItemLink(i);
-- 	link = GetBootyItemLink(i);
-- 	GameTooltip:SetHyperLink(link);
-- 	local i=1;
-- 	while true do
-- 		local text = getglobal("GameTooltipTextLeft"..i):GetText();
-- 		if (text == nil or i>20) then
-- 			break;
-- 		else
-- 			DEFAULT_CHAT_FRAME:AddMessage(i..": "..text);
-- 		end;
-- 		i = i+1;
-- 	end;
-- end;

----------------------------------------------------------------------------------------------------
-------------------- New content/behaviour of original frames --------------------------------------
----------------------------------------------------------------------------------------------------

function Lootomatic_RollOnLoot(frame,mode)
 	local name, icon, itemCount, quality = GetLootRollItemInfo(frame);
 	if (name ~= nil and IsCtrlKeyDown()) then
		if (IsShiftKeyDown()==true) then
			for i=1,table.maxn(Lootomatic_Settings.ItemFilter) do
				if (Lootomatic_Settings.ItemFilter[i]["Name"] == name and Lootomatic_Settings.ItemFilter[i]["Char"] == UnitName("player")) then
					table.remove(Lootomatic_Settings.ItemFilter,i);
				end;
			end;

			local rollmode=1;
			    if (mode=="roll")  then rollmode=4;
			elseif (mode=="greed") then rollmode=3;
			                       else rollmode=2; end;

			table.insert(Lootomatic_Settings.ItemFilter,
				{
				Name = name,
				Loot = 1,
				Roll = rollmode,
				Char = UnitName("player"),
				});
			table.sort(Lootomatic_Settings.ItemFilter,Lootomatic_ItemFilter_ListSort);
			DEFAULT_CHAT_FRAME:AddMessage("|cffFFFE80["..Lootomatic.Name.."]:|r "..string.gsub(string.gsub(Lootomatic.Strings.AddQuickFilter2,"#ITEM#",tostring(name)),"#ROLL#",Lootomatic.Strings.RollStatus[rollmode]),0.7,0.7,0.7);
		else
			for i=1,table.maxn(Lootomatic_Settings.ItemFilter) do
				if (Lootomatic_Settings.ItemFilter[i]["Name"] == name and Lootomatic_Settings.ItemFilter[i]["Char"] == nil) then
					table.remove(Lootomatic_Settings.ItemFilter,i);
				end;
			end;

			local rollmode=1;
			    if (mode=="roll")  then rollmode=4;
			elseif (mode=="greed") then rollmode=3;
			                       else rollmode=2; end;

			table.insert(Lootomatic_Settings.ItemFilter,
				{
				Name = name,
				Loot = 1,
				Roll = rollmode,
				});
			table.sort(Lootomatic_Settings.ItemFilter,Lootomatic_ItemFilter_ListSort);
			DEFAULT_CHAT_FRAME:AddMessage("|cffFFFE80["..Lootomatic.Name.."]:|r "..string.gsub(string.gsub(Lootomatic.Strings.AddQuickFilter1,"#ITEM#",tostring(name)),"#ROLL#",Lootomatic.Strings.RollStatus[rollmode]),0.7,0.7,0.7);
		end;
	end;
 	Sol.hooks.GetOriginalFn("Lootomatic", "RollOnLoot") (frame, mode)
end;

function Lootomatic_RollOnLoot_Tooltips()
	if (Lootomatic_Settings.LongTooltips) then
		GROUPLOOT_ROLL   = "|cff00c0f3"..LootomaticBackup_GROUPLOOT_ROLL  .."|r\n|cffFFFE80["..Lootomatic.Name.."]:|r "..Lootomatic.Strings.RollFrameSave;
		GROUPLOOT_GREED  = "|cff00c0f3"..LootomaticBackup_GROUPLOOT_GREED .."|r\n|cffFFFE80["..Lootomatic.Name.."]:|r "..Lootomatic.Strings.RollFrameSave;
		GROUPLOOT_CANCEL = "|cff00c0f3"..LootomaticBackup_GROUPLOOT_CANCEL.."|r\n|cffFFFE80["..Lootomatic.Name.."]:|r "..Lootomatic.Strings.RollFrameSave;
	else
		GROUPLOOT_ROLL   = "|cff00c0f3"..LootomaticBackup_GROUPLOOT_ROLL  .."|r\n|cffFFFE80["..Lootomatic.Name.."]";
		GROUPLOOT_GREED  = "|cff00c0f3"..LootomaticBackup_GROUPLOOT_GREED .."|r\n|cffFFFE80["..Lootomatic.Name.."]";
		GROUPLOOT_CANCEL = "|cff00c0f3"..LootomaticBackup_GROUPLOOT_CANCEL.."|r\n|cffFFFE80["..Lootomatic.Name.."]";
	end;
end;

function Lootomatic_BagItemButton_OnClick(frame, button, ignoreShift)
	if (button=="LBUTTON" and IsShiftKeyDown() and Lootomatic_ItemFilterFrame:IsVisible()) then
		local index, file, name, itemCount, locked = GetBagItemInfo(frame:GetID());
		if (name) then Lootomatic_ItemFilter_EnterName(name); end;
	else
		Sol.hooks.GetOriginalFn("Lootomatic", "BagItemButton_OnClick") (frame, button, ignoreShift)
	end;
end;

function Lootomatic_Hyperlink_Assign(link, key)
	Sol.hooks.GetOriginalFn("Lootomatic", "Hyperlink_Assign")(link,key);

	local _type, _data, _name = ParseHyperlink(link);
	if (_type=="item" and key == "RBUTTON" ) then
  		local info = {};
		info.text = "|cffFFFE80[Lootomatic]:|r "..Lootomatic.Strings.AddFilter;
		info.func = function() Lootomatic_ItemFilter_EnterName(_name); end;
		UIDropDownMenu_AddButton(info, 1);
		UIDropDownMenu_Refresh(ChatFrameDropDown);
	end;
end;

function Lootomatic_ChatFrame_OnEvent(this, event)
	if ( event == "CHAT_MSG_SYSTEM" ) then
		if (string.find(arg1, "^%[[^%]]*%] : ") ~= nil) then
			itemstring = string.gsub(string.gsub(arg1,"^%[[^%]]*%] : .*%|c",""),"%|r.*$","");

			for n=1,6 do
				if (string.find(itemstring,"^ff"..Lootomatic.QualityColors[n])~=nil and Lootomatic_Settings.Chatfilter[n]==false) then return; end;
			end;
		end;
	end;
	Sol.hooks.GetOriginalFn("Lootomatic", "ChatFrame_OnEvent")(this, event);
end;




----------------------------------------------------------------------------------------------------
-------------------- Minimap-Menue -----------------------------------------------------------------
----------------------------------------------------------------------------------------------------

function Lootomatic_MainMenue()
	if (UIDROPDOWNMENU_MENU_LEVEL == 1) then
		local menuitem = {};
		menuitem.func = Lootomatic_Toggle;
		if (Lootomatic_Settings.Enabled==true) then menuitem.text = Lootomatic.Strings.MEnabled;  menuitem.checked = 1;
		                                       else menuitem.text = Lootomatic.Strings.MDisabled; menuitem.checked = nil; end;
		UIDropDownMenu_AddButton(menuitem,UIDROPDOWNMENU_MENU_LEVEL);

		local menuitem = {};
		menuitem.notCheckable = 1;

		menuitem.value = 1;
		if (IsPartyLeader("player")~=true) then menuitem.disabled = true; else menuitem.hasArrow = 1; end;
		menuitem.text = Lootomatic.Strings.Roll     ..": "..Lootomatic.ListEntries[GetLootThreshold()+2];
		UIDropDownMenu_AddButton(menuitem,UIDROPDOWNMENU_MENU_LEVEL);

		menuitem.value = 2;
		menuitem.hasArrow = 1;
		menuitem.disabled = nil;
		menuitem.text = Lootomatic.Strings.AutoLoot ..": "..Lootomatic.ListEntries[Lootomatic_Settings.AutoLoot];
		UIDropDownMenu_AddButton(menuitem,UIDROPDOWNMENU_MENU_LEVEL);

		menuitem.value = 3;
		menuitem.text = Lootomatic.Strings.AutoPass ..": "..Lootomatic.ListEntries[Lootomatic_Settings.AutoPass];
		UIDropDownMenu_AddButton(menuitem,UIDROPDOWNMENU_MENU_LEVEL);

		menuitem.value = 4;
		menuitem.text = Lootomatic.Strings.AutoGreed..": "..Lootomatic.ListEntries[Lootomatic_Settings.AutoGreed];
		UIDropDownMenu_AddButton(menuitem,UIDROPDOWNMENU_MENU_LEVEL);

		local menuitem = {};
		menuitem.notCheckable = 1;
		menuitem.text = Lootomatic.Strings.ItemFilter;
		menuitem.func = Lootomatic_ItemFilter_Toggle;
		UIDropDownMenu_AddButton(menuitem,UIDROPDOWNMENU_MENU_LEVEL);

		menuitem.text = Lootomatic.Strings.Settings;
		menuitem.func = Lootomatic_Settings_Toggle;
		UIDropDownMenu_AddButton(menuitem,UIDROPDOWNMENU_MENU_LEVEL);

		menuitem.hasArrow = 1;
		menuitem.value = 5;
		menuitem.text = Lootomatic.Strings.RollMessages;
		UIDropDownMenu_AddButton(menuitem,UIDROPDOWNMENU_MENU_LEVEL);
	else
		local menuitem = {};
		if (UIDROPDOWNMENU_MENU_VALUE == 1) then
			for i=2,7 do
				menuitem.text = Lootomatic.ListEntries[i];
				menuitem.func = function() SetLootMethod("alternate"); SetLootThreshold(i-2); CloseDropDownMenus(1); ToggleDropDownMenu(Lootomatic_Dropdown); end;
				if (GetLootThreshold()+2==i) then menuitem.checked = 1;
				                             else menuitem.checked = nil; end;
				UIDropDownMenu_AddButton(menuitem,UIDROPDOWNMENU_MENU_LEVEL);
			end;
			menuitem.text = Lootomatic.ListEntries[101];
			menuitem.func = function() SetLootMethod("freeforall"); CloseDropDownMenus(1); ToggleDropDownMenu(Lootomatic_Dropdown); end;
			if (GetLootThreshold()==99) then menuitem.checked = 1;
			                            else menuitem.checked = nil; end;
			UIDropDownMenu_AddButton(menuitem,UIDROPDOWNMENU_MENU_LEVEL);
		elseif (UIDROPDOWNMENU_MENU_VALUE == 2) then
			for i=1,7 do
				menuitem.text = Lootomatic.ListEntries[i];
				menuitem.func = function() Lootomatic_Settings.AutoLoot = i; CloseDropDownMenus(1); ToggleDropDownMenu(Lootomatic_Dropdown); end;
				if (Lootomatic_Settings.AutoLoot==i) then menuitem.checked = 1;
				                                     else menuitem.checked = nil; end;
				UIDropDownMenu_AddButton(menuitem,UIDROPDOWNMENU_MENU_LEVEL);
			end;
			menuitem = {};
			menuitem.isTitle = true;
			UIDropDownMenu_AddButton(menuitem,UIDROPDOWNMENU_MENU_LEVEL);

			menuitem = {};
			menuitem.text = Lootomatic.Strings.NoBoss;
			menuitem.func = function() Lootomatic_Settings.AutoLootBoss = not Lootomatic_Settings.AutoLootBoss; CloseDropDownMenus(1); ToggleDropDownMenu(Lootomatic_Dropdown); end;
			if (Lootomatic_Settings.AutoLootBoss==false) then menuitem.checked = 1;
			                                             else menuitem.checked = nil; end;
			UIDropDownMenu_AddButton(menuitem,UIDROPDOWNMENU_MENU_LEVEL);
		elseif (UIDROPDOWNMENU_MENU_VALUE == 3) then
			for i=1,7 do
				menuitem.text = Lootomatic.ListEntries[i];
				menuitem.func = function() Lootomatic_Settings.AutoPass = i; CloseDropDownMenus(1); ToggleDropDownMenu(Lootomatic_Dropdown); end;
				if (Lootomatic_Settings.AutoPass==i) then menuitem.checked = 1;
				                                     else menuitem.checked = nil; end;
				UIDropDownMenu_AddButton(menuitem,UIDROPDOWNMENU_MENU_LEVEL);
			end;
		elseif (UIDROPDOWNMENU_MENU_VALUE == 4) then
			for i=1,7 do
				menuitem.text = Lootomatic.ListEntries[i];
				menuitem.func = function() Lootomatic_Settings.AutoGreed = i; CloseDropDownMenus(1); ToggleDropDownMenu(Lootomatic_Dropdown); end;
				if (Lootomatic_Settings.AutoGreed==i) then menuitem.checked = 1;
				                                      else menuitem.checked = nil; end;
				UIDropDownMenu_AddButton(menuitem,UIDROPDOWNMENU_MENU_LEVEL);
			end;
		elseif (UIDROPDOWNMENU_MENU_VALUE == 5) then
			for i=2,7 do
				menuitem.text = Lootomatic.ListEntries[i];
				menuitem.func = function() Lootomatic_Settings.Chatfilter[i-1] = not Lootomatic_Settings.Chatfilter[i-1]; CloseDropDownMenus(1); ToggleDropDownMenu(Lootomatic_Dropdown); end;
				if (Lootomatic_Settings.Chatfilter[i-1]==true) then menuitem.checked = 1;
				                                               else menuitem.checked = nil; end;
				UIDropDownMenu_AddButton(menuitem,UIDROPDOWNMENU_MENU_LEVEL);
			end;
		end;
	end;
end;





----------------------------------------------------------------------------------------------------
-------------------- Miscallaneous -----------------------------------------------------------------
----------------------------------------------------------------------------------------------------

function Lootomatic_Tooltip(Object)
	GameTooltip:SetOwner(Object,"ANCHOR_BOTTOM",0,0);

	if (Object:GetName()=="Lootomatic_Minimap") then
		GameTooltip:SetText("|cffFFFE80"..Lootomatic.Name.." "..Lootomatic.Version.."|r");

		if (Lootomatic_Settings.Enabled == true) then GameTooltip:AddLine("   "..Lootomatic.Strings.Enabled ,0,1,0);
		                                         else GameTooltip:AddLine("   "..Lootomatic.Strings.Disabled,1,0,0); end;

		local bosses = "";
		if (Lootomatic_Settings.AutoLootBoss == false) then bosses = ", "..Lootomatic.Strings.NoBoss; end;

		GameTooltip:AddLine("   "..Lootomatic.Strings.Roll     ..": "..Lootomatic.ListEntries[Lootomatic_Settings.Roll]);
		GameTooltip:AddLine("   "..Lootomatic.Strings.AutoLoot ..": "..Lootomatic.ListEntries[Lootomatic_Settings.AutoLoot]..bosses);
		GameTooltip:AddLine("   "..Lootomatic.Strings.AutoPass ..": "..Lootomatic.ListEntries[Lootomatic_Settings.AutoPass]);
		GameTooltip:AddLine("   "..Lootomatic.Strings.AutoGreed..": "..Lootomatic.ListEntries[Lootomatic_Settings.AutoGreed]);

		local messages = "";
		for n=1,6 do
			if (Lootomatic_Settings.Chatfilter[n]==true) then messages=messages.." |cff"..Lootomatic.QualityColors[n].."+|r"; end;
		end;
		GameTooltip:AddLine("   "..Lootomatic.Strings.RollMessages..":"..messages);

		GameTooltip:AddLine(UI_MINIMAPBUTTON_MOVE, 0, 0.75, 0.95);
	else
		GameTooltip:SetText("|cffFFFE80"..Lootomatic.Strings["Tooltip_"..Object:GetName()][1].."|r");
		GameTooltip:AddLine(              Lootomatic.Strings["Tooltip_"..Object:GetName()][2]      );

		if (table.maxn(Lootomatic.Strings["Tooltip_"..Object:GetName()])>2) then
			if (IsShiftKeyDown()) then GameTooltip:AddLine(Lootomatic.Strings["Tooltip_"..Object:GetName()][3]);
			                      else GameTooltip:AddLine(Lootomatic.Strings["Tooltip_LongText"], 0, 0.75, 0.95); end;
		end;
	end;
end;

function Lootomatic_RollFrame_Slider()
	Lootomatic_Settings.Interval = (Lootomatic_SettingsFrame_Rollframe_Slider:GetValue())/10;
	Lootomatic_SettingsFrame_Rollframe_Slider_Value:SetText(Lootomatic.Strings.Settings_RollPositionTimer..": "..string.format("%0.1f",(Lootomatic_SettingsFrame_Rollframe_Slider:GetValue())/10).." s");
end;
function Lootomatic_RollFrameGhosts_Slider()
	if (math.abs(Lootomatic_RollFrameGhosts_1_Slider:GetValue()) < 5) then Lootomatic_RollFrameGhosts_1_Slider:SetValue(0); end;
	Lootomatic_RollFrameGhosts_1_Slider_Value:SetText(tostring(Lootomatic_RollFrameGhosts_1_Slider:GetValue()));
	Lootomatic_RollFrameGhosts_Move();
end;

function Lootomatic_RollFrameGhosts_Move()
	Lootomatic_RollFrameGhosts_2:ClearAllAnchors();
	Lootomatic_RollFrameGhosts_3:ClearAllAnchors();
	Lootomatic_RollFrameGhosts_4:ClearAllAnchors();
	Lootomatic_RollFrameGhosts_5:ClearAllAnchors();
	Lootomatic_RollFrameGhosts_6:ClearAllAnchors();

	Lootomatic_RollFrameGhosts_2:SetAnchor("CENTER", "CENTER", Lootomatic_RollFrameGhosts_1, 0, Lootomatic_RollFrameGhosts_1_Slider:GetValue());
	Lootomatic_RollFrameGhosts_3:SetAnchor("CENTER", "CENTER", Lootomatic_RollFrameGhosts_2, 0, Lootomatic_RollFrameGhosts_1_Slider:GetValue());
	Lootomatic_RollFrameGhosts_4:SetAnchor("CENTER", "CENTER", Lootomatic_RollFrameGhosts_3, 0, Lootomatic_RollFrameGhosts_1_Slider:GetValue());
	Lootomatic_RollFrameGhosts_5:SetAnchor("CENTER", "CENTER", Lootomatic_RollFrameGhosts_4, 0, Lootomatic_RollFrameGhosts_1_Slider:GetValue());
	Lootomatic_RollFrameGhosts_6:SetAnchor("CENTER", "CENTER", Lootomatic_RollFrameGhosts_5, 0, Lootomatic_RollFrameGhosts_1_Slider:GetValue());
end;

function Lootomatic_Credits_Toggle()
	if (Lootomatic.Fader==0) then
		if (Lootomatic_Credits:IsVisible()) then
			Lootomatic.Fader = -1;
		else
			Lootomatic_Credits:SetAlpha(0);
			Lootomatic_Credits:Show();
			Lootomatic.Fader = 1;
		end;
	end;
end;
function Lootomatic_Settings_Toggle()
	if (Lootomatic_SettingsFrame:IsVisible()) then Lootomatic_SettingsFrame:Hide();
		                                  else Lootomatic_SettingsFrame:Show();end;
end;
function Lootomatic_Toggle()
	if (Lootomatic_Settings.Enabled == true) then Lootomatic_Settings.Enabled = false;
	                                         else Lootomatic_Settings.Enabled = true; end;
end;
function Lootomatic_Enable()
	Lootomatic_Settings.AM = true;
	Lootomatic_Minimap:Show();
end;
function Lootomatic_Disable()
	Lootomatic_Settings.AM = false;
	Lootomatic_Minimap:Hide();
end;





----------------------------------------------------------------------------------------------------
-------------------- Item Filter -------------------------------------------------------------------
----------------------------------------------------------------------------------------------------

function Lootomatic_ItemFilter_Repaint()
	local a,b = Lootomatic_ItemFilterFrame_Filterlist:GetSize();
	local maxvalues = math.floor((b-10)/20);

	local maxvalue = Lootomatic_ItemFilter_GetDisplayIndex(table.maxn(Lootomatic_Settings.ItemFilter))-maxvalues+1;

	if (maxvalue < 1) then maxvalue = 1; end;
	if (maxvalue== 1) then Lootomatic_ItemFilterFrame_Filterlist_ScrollBar:Hide();
	                  else Lootomatic_ItemFilterFrame_Filterlist_ScrollBar:Show(); end;

	Lootomatic_ItemFilterFrame_Filterlist_ScrollBar:SetMinMaxValues(1,maxvalue);

	local value = Lootomatic_ItemFilterFrame_Filterlist_ScrollBar:GetValue();

	for i=1,30 do
		local index = Lootomatic_ItemFilter_GetRealIndex(value-1+i);

		if (index <= table.maxn(Lootomatic_Settings.ItemFilter) and i <= maxvalues) then
			getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i):Show();
			getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Name"):SetText(tostring(                              Lootomatic_Settings.ItemFilter[index].Name ));
			getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Loot"):SetText(tostring(Lootomatic.Strings.LootStatus[Lootomatic_Settings.ItemFilter[index].Loot]));
			getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Roll"):SetText(tostring(Lootomatic.Strings.RollStatus[Lootomatic_Settings.ItemFilter[index].Roll]));

			if (Lootomatic_Settings.ItemFilter[index].Char == nil) then
				getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Name"):SetColor(1,1,1);
				getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Loot"):SetColor(1,1,1);
				getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Roll"):SetColor(1,1,1);
			else
				getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Name"):SetColor(0.5,0.5,0.5);
				getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Loot"):SetColor(0.5,0.5,0.5);
				getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Roll"):SetColor(0.5,0.5,0.5);
			end;
		else
			getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i):Hide();
			getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Name"):SetText("");
			getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Loot"):SetText("");
			getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Roll"):SetText("");
		end;
	end;

	Lootomatic_ItemFilter_SetValues();
end;

function Lootomatic_ItemFilter_GetRealIndex(x)
	local n=0;
	for i=1,table.maxn(Lootomatic_Settings.ItemFilter) do
		if (Lootomatic_Settings.ItemFilter[i].Char==nil
		or  Lootomatic_Settings.ItemFilter[i].Char==UnitName("player")) then n=n+1; end;
		if (n==x) then return i; end;
	end;
	return x;
end;
function Lootomatic_ItemFilter_GetDisplayIndex(x)
	local n=0;
	for i=1,x do
		if (Lootomatic_Settings.ItemFilter[i].Char~=nil
		and Lootomatic_Settings.ItemFilter[i].Char~=UnitName("player")) then n=n+1; end;
	end;
	return (x-n);
end;
function Lootomatic_ItemFilter_RowClicked(id)
	Lootomatic.ListSelect = Lootomatic_ItemFilter_GetRealIndex(Lootomatic_ItemFilterFrame_Filterlist_ScrollBar:GetValue()+id-1);
	Lootomatic_ItemFilter_SetValues();
end;

function Lootomatic_ItemFilter_SearchName()
	Lootomatic.ListSelect = nil;
	for i=1,table.maxn(Lootomatic_Settings.ItemFilter) do
		if (Lootomatic_Settings.ItemFilter[i]["Name"] == Lootomatic_ItemFilterFrame_Itemname:GetText()) then
			Lootomatic.ListSelect = i;
			break;
		end;
	end;
end;

function Lootomatic_ItemFilter_CheckEntries()
	local found1=false;
	local found2=false;
	for i=1,table.maxn(Lootomatic_Settings.ItemFilter) do
		if (Lootomatic_Settings.ItemFilter[i]["Name"] == Lootomatic_ItemFilterFrame_Itemname:GetText()) then
			    if (Lootomatic_Settings.ItemFilter[i].Char==nil               ) then found1=i;
			elseif (Lootomatic_Settings.ItemFilter[i].Char==UnitName("player")) then found2=i;
			end;
		end;
	end;
	return found1,found2;
end;

function Lootomatic_ItemFilter_SetValues()
	if (Lootomatic.ListSelect == nil) then
		Lootomatic_ItemFilterFrame_Itemname_Text :SetColor(1,1,1);
		Lootomatic_ItemFilterFrame_Filterlist_Highlight:Hide();
	else
		Lootomatic_ItemFilterFrame_Itemname_Text :SetColor(1,0.82,0);
		Lootomatic_ItemFilterFrame_Itemname:SetText(Lootomatic_Settings.ItemFilter[Lootomatic.ListSelect]["Name"]);

		UIDropDownMenu_SetSelectedID(Lootomatic_ItemFilterFrame_Looting, Lootomatic_Settings.ItemFilter[Lootomatic.ListSelect]["Loot"]);
		UIDropDownMenu_SetText      (Lootomatic_ItemFilterFrame_Looting, Lootomatic.Strings.LootStatus[Lootomatic_Settings.ItemFilter[Lootomatic.ListSelect]["Loot"]]);

		UIDropDownMenu_SetSelectedID(Lootomatic_ItemFilterFrame_Rolling, Lootomatic_Settings.ItemFilter[Lootomatic.ListSelect]["Roll"]);
		UIDropDownMenu_SetText      (Lootomatic_ItemFilterFrame_Rolling, Lootomatic.Strings.RollStatus[Lootomatic_Settings.ItemFilter[Lootomatic.ListSelect]["Roll"]]);

		local a,b       = Lootomatic_ItemFilterFrame_Filterlist:GetSize();
		local maxvalues = math.floor((b-10)/20);
		local offset    = Lootomatic_ItemFilter_GetDisplayIndex(Lootomatic.ListSelect)-Lootomatic_ItemFilterFrame_Filterlist_ScrollBar:GetValue();

		if (offset>=0 and offset<maxvalues) then
			Lootomatic_ItemFilterFrame_Filterlist_Highlight:ClearAllAnchors();
			Lootomatic_ItemFilterFrame_Filterlist_Highlight:SetAnchor("TOPLEFT", "TOPLEFT", Lootomatic_ItemFilterFrame_Filterlist, 5, 9+offset*20);
			if (Lootomatic_ItemFilterFrame_Filterlist_ScrollBar:IsVisible()) then a=a-15; end;
			Lootomatic_ItemFilterFrame_Filterlist_Highlight:SetSize(a-10,22);
			Lootomatic_ItemFilterFrame_Filterlist_Highlight:Show();
		else
			Lootomatic_ItemFilterFrame_Filterlist_Highlight:Hide();
		end;
	end;

	if (string.len(Lootomatic_ItemFilterFrame_Itemname:GetText())==0 or (UIDropDownMenu_GetSelectedID(Lootomatic_ItemFilterFrame_Looting) == 1 and UIDropDownMenu_GetSelectedID(Lootomatic_ItemFilterFrame_Rolling) == 1)) then
		Lootomatic_ItemFilterFrame_ItemAdd:Disable();
	else
		Lootomatic_ItemFilterFrame_ItemAdd:Enable();
	end;

	if (Lootomatic.ListSelect == nil) then
		Lootomatic_ItemFilterFrame_ItemRemove:Disable();
	else
		Lootomatic_ItemFilterFrame_ItemRemove:Enable();
	end;
end;





function Lootomatic_ItemFilter_AddItem(key)
	local found1,found2 = Lootomatic_ItemFilter_CheckEntries();

	    if (found1~=false  and found2~=false) then DEFAULT_CHAT_FRAME:AddMessage("|cffFFFE80["..Lootomatic.Name.."]:|r "..Lootomatic.Strings.Added3,0.7,0.7,0.7);
	elseif (key=="LBUTTON" and found1~=false) then DEFAULT_CHAT_FRAME:AddMessage("|cffFFFE80["..Lootomatic.Name.."]:|r "..Lootomatic.Strings.Added1,0.7,0.7,0.7);
	elseif (key=="RBUTTON" and found2~=false) then DEFAULT_CHAT_FRAME:AddMessage("|cffFFFE80["..Lootomatic.Name.."]:|r "..Lootomatic.Strings.Added2,0.7,0.7,0.7);
	else
		if (key=="LBUTTON") then
			table.insert(Lootomatic_Settings.ItemFilter,
				{
				Name = Lootomatic_ItemFilterFrame_Itemname:GetText(),
				Loot = UIDropDownMenu_GetSelectedID(Lootomatic_ItemFilterFrame_Looting),
				Roll = UIDropDownMenu_GetSelectedID(Lootomatic_ItemFilterFrame_Rolling),
				});
		elseif (key=="RBUTTON") then
			table.insert(Lootomatic_Settings.ItemFilter,
				{
				Name = Lootomatic_ItemFilterFrame_Itemname:GetText(),
				Loot = UIDropDownMenu_GetSelectedID(Lootomatic_ItemFilterFrame_Looting),
				Roll = UIDropDownMenu_GetSelectedID(Lootomatic_ItemFilterFrame_Rolling),
				Char = UnitName("player"),
				});
		end;
		table.sort(Lootomatic_Settings.ItemFilter,Lootomatic_ItemFilter_ListSort);

		found1,found2 = Lootomatic_ItemFilter_CheckEntries();

		    if (key=="LBUTTON") then Lootomatic.ListSelect = found1;
		elseif (key=="RBUTTON") then Lootomatic.ListSelect = found2;
		end;
	end;

	Lootomatic_ItemFilter_Repaint();
end;

function Lootomatic_ItemFilter_RemoveItem()
	if (Lootomatic.ListSelect ~= nil) then
		table.remove(Lootomatic_Settings.ItemFilter,Lootomatic.ListSelect);
		Lootomatic.ListSelect = nil;
		Lootomatic_ItemFilter_Repaint();
	end;
end;

function Lootomatic_ItemFilter_ListSort(a,b)
	    if (string.lower(string.gsub(a.Name,"[\* ]","")) == string.lower(string.gsub(b.Name,"[\* ]",""))) then
		if (a.Char == nil and b.Char ~= nil) then return true;
		else return false; end;
	elseif (string.lower(string.gsub(a.Name,"[\* ]","")) <  string.lower(string.gsub(b.Name,"[\* ]",""))) then return true;
	else return false;
	end;
end;





function Lootomatic_ItemFilter_EnterName(name)
	Lootomatic.ListSelect = nil;
	Lootomatic_ItemFilterFrame:Show();
	Lootomatic_ItemFilterFrame_Itemname:SetText(name);

	UIDropDownMenu_SetSelectedID(Lootomatic_ItemFilterFrame_Looting, 1);
	UIDropDownMenu_SetText      (Lootomatic_ItemFilterFrame_Looting, Lootomatic.Strings.LootStatus[1]);

	UIDropDownMenu_SetSelectedID(Lootomatic_ItemFilterFrame_Rolling, 1);
	UIDropDownMenu_SetText      (Lootomatic_ItemFilterFrame_Rolling, Lootomatic.Strings.RollStatus[1]);

	Lootomatic_ItemFilter_SetValues();
end;

function Lootomatic_ItemFilter_LootmenuItems()
	local menuitem;

	menuitem = {};
	menuitem.text = Lootomatic.Strings.LootStatus[1];
	menuitem.func = function() Lootomatic_ItemFilter_LootmenuChange(1); end;
	UIDropDownMenu_AddButton(menuitem,1);

	menuitem = {};
	menuitem.text = Lootomatic.Strings.LootStatus[2];
	menuitem.func = function() Lootomatic_ItemFilter_LootmenuChange(2); end;
	UIDropDownMenu_AddButton(menuitem,1);

	menuitem = {};
	menuitem.text = Lootomatic.Strings.LootStatus[3];
	menuitem.func = function() Lootomatic_ItemFilter_LootmenuChange(3); end;
	UIDropDownMenu_AddButton(menuitem,1);

	menuitem = {};
	menuitem.text = Lootomatic.Strings.LootStatus[4];
	menuitem.func = function() Lootomatic_ItemFilter_LootmenuChange(4); end;
	UIDropDownMenu_AddButton(menuitem,1);
end;
function Lootomatic_ItemFilter_LootmenuChange(set)
	if (Lootomatic.ListSelect ~= nil) then Lootomatic_Settings.ItemFilter[Lootomatic.ListSelect]["Loot"] = set; end;
	UIDropDownMenu_SetSelectedID(Lootomatic_ItemFilterFrame_Looting, set);
	Lootomatic_ItemFilter_Repaint();
end;

function Lootomatic_ItemFilter_RollmenuItems()
	local menuitem;

	menuitem = {};
	menuitem.text = Lootomatic.Strings.RollStatus[1];
	menuitem.func = function() Lootomatic_ItemFilter_RollmenuChange(1); end;
	UIDropDownMenu_AddButton(menuitem,1);

	menuitem = {};
	menuitem.text = Lootomatic.Strings.RollStatus[2];
	menuitem.func = function() Lootomatic_ItemFilter_RollmenuChange(2); end;
	UIDropDownMenu_AddButton(menuitem,1);

	menuitem = {};
	menuitem.text = Lootomatic.Strings.RollStatus[3];
	menuitem.func = function() Lootomatic_ItemFilter_RollmenuChange(3); end;
	UIDropDownMenu_AddButton(menuitem,1);

	menuitem = {};
	menuitem.text = Lootomatic.Strings.RollStatus[4];
	menuitem.func = function() Lootomatic_ItemFilter_RollmenuChange(4); end;
	UIDropDownMenu_AddButton(menuitem,1);

	menuitem = {};
	menuitem.text = Lootomatic.Strings.RollStatus[5];
	menuitem.func = function() Lootomatic_ItemFilter_RollmenuChange(5); end;
	UIDropDownMenu_AddButton(menuitem,1);
end;
function Lootomatic_ItemFilter_RollmenuChange(set)
	if (Lootomatic.ListSelect ~= nil) then Lootomatic_Settings.ItemFilter[Lootomatic.ListSelect]["Roll"] = set; end;
	UIDropDownMenu_SetSelectedID(Lootomatic_ItemFilterFrame_Rolling, set);
	Lootomatic_ItemFilter_Repaint();
end;

function Lootomatic_ItemFilter_Highlight(Object,mode)
	if (mode==true) then
		local a = getglobal(Object.."_Name"):GetColor();
		if (a==1) then
			getglobal(Object.."_Name"):SetColor(1,0.82,0);
			getglobal(Object.."_Loot"):SetColor(1,0.82,0);
			getglobal(Object.."_Roll"):SetColor(1,0.82,0);
		else
			getglobal(Object.."_Name"):SetColor(0.5,0.41,0);
			getglobal(Object.."_Loot"):SetColor(0.5,0.41,0);
			getglobal(Object.."_Roll"):SetColor(0.5,0.41,0);
		end;
	else
		Lootomatic_ItemFilter_Repaint();
		local a = getglobal(Object.."_Name"):GetColor();
		if (a==1) then
			getglobal(Object.."_Name"):SetColor(1,1,1);
			getglobal(Object.."_Loot"):SetColor(1,1,1);
			getglobal(Object.."_Roll"):SetColor(1,1,1);
		else
			getglobal(Object.."_Name"):SetColor(0.5,0.5,0.5);
			getglobal(Object.."_Loot"):SetColor(0.5,0.5,0.5);
			getglobal(Object.."_Roll"):SetColor(0.5,0.5,0.5);
		end;
	end;
end;

function Lootomatic_ItemFilter_ReceiveDrag(this)
	if ( CursorItemType() == "bag" ) then
		local id = GetCursorItemInfo();
		local icon, name, count = GetGoodsItemInfo(id);
		Lootomatic_ItemFilter_EnterName(name);
		Lootomatic_ItemFilterFrame_Itemname:ClearFocus();
		InsertGarbageItem(id, true);
		RemoveGarbageItem(id);
	end
end;

function Lootomatic_ItemFilter_Resize()
	local a,b = Lootomatic_ItemFilterFrame:GetPos();
	local c,d = Lootomatic_ItemFilterFrame_ResizeButton:GetPos();

	local x = (c-a+19)/GetUIScale();
	local y = (d-b+19)/GetUIScale();

	if (x<400) then x = 400; end;
	if (y<210) then y = 210; end;

	Lootomatic_ItemFilterFrame:SetSize(x,y);
	Lootomatic_ItemFilterFrame:ClearAllAnchors();
	Lootomatic_ItemFilterFrame:SetAnchor("TOPLEFT", "TOPLEFT", UIParent, a/GetUIScale(), b/GetUIScale());
	Lootomatic_ItemFilterFrame_ResizeButton:ClearAllAnchors();
	Lootomatic_ItemFilterFrame_ResizeButton:SetAnchor("BOTTOMRIGHT", "BOTTOMRIGHT", Lootomatic_ItemFilterFrame, -3, -3);

	Lootomatic_ItemFilter_MaxSize();
	Lootomatic_ItemFilter_ResizeElements();

	Lootomatic_Settings.ItemFilterX,Lootomatic_Settings.ItemFilterY = Lootomatic_ItemFilterFrame:GetSize();
end;

function Lootomatic_ItemFilter_MaxSize()
	local update=false;
	local a,b = UIParent:GetSize();
	local c,d = Lootomatic_ItemFilterFrame:GetSize();

	if (c > a-50) then c=a-50; update=true; end;
	if (d > b-50) then d=b-50; update=true; end;

	if (update) then Lootomatic_ItemFilterFrame:SetSize(c,d); end;
end;

function Lootomatic_ItemFilter_ResizeElements()
	local a,b = Lootomatic_ItemFilterFrame:GetSize();

	Lootomatic_ItemFilterFrame_Header:SetSize(a,26);
	Lootomatic_ItemFilterFrame_Itemname:SetSize(a-290,28);
	Lootomatic_ItemFilterFrame_Filterlist:SetSize(a-40,b-90);
	Lootomatic_ItemFilterFrame_Filterlist_ScrollBar:SetSize(10,b-138);

	for i=1,30 do
		getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i):SetSize(a-60,20);
		getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Name"):SetSize(a-250,20);
		getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Loot"):ClearAllAnchors();
		getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Roll"):ClearAllAnchors();
		getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Loot"):SetAnchor("LEFT", "LEFT", getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i), a-230, 0);
		getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i.."_Roll"):SetAnchor("LEFT", "LEFT", getglobal("Lootomatic_ItemFilterFrame_Filterlist_Item"..i), a-125, 0);
	end;

	Lootomatic_ItemFilter_Repaint();
end;

function Lootomatic_ItemFilter_Scroll(delta)
	    if (delta>0) then Lootomatic_ItemFilterFrame_Filterlist_ScrollBar:SetValue(Lootomatic_ItemFilterFrame_Filterlist_ScrollBar:GetValue()-1);
	elseif (delta<0) then Lootomatic_ItemFilterFrame_Filterlist_ScrollBar:SetValue(Lootomatic_ItemFilterFrame_Filterlist_ScrollBar:GetValue()+1); end;
end;

function Lootomatic_ItemFilter_Toggle()
	if (Lootomatic_ItemFilterFrame:IsVisible()) then Lootomatic_ItemFilterFrame:Hide();
		                                    else Lootomatic_ItemFilterFrame:Show(); end;
end;





-- eof -- Lootomatic, written by PetraAreon