yGather.settings = {};

-- this package needs following globals
local _G = _G;
local SaveVariables = SaveVariables;
local pairs = pairs;

setfenv(1, yGather.settings);

local current = {};
local temporary = {};
local default = {
    ["worldmap/showResources"] = true;
    ["worldmap/iconMode"] = "coloredDots";
    ["worldmap/iconOreColor"] = {1, 1, 0};
    ["worldmap/iconHerbsColor"] = {0, 1, 0};
    ["worldmap/iconWoodColor"] = {1, 0.5, 0};
    ["worldmap/iconTransparency"] = 0;
    ["worldmap/iconSize"] = 6;
    
    ["minimap/showResources"] = true;
    ["minimap/iconMode"] = "coloredDots";
    ["minimap/iconOreColor"] = {1, 1, 0};
    ["minimap/iconHerbsColor"] = {0, 1, 0};
    ["minimap/iconWoodColor"] = {1, 0.5, 0};
    ["minimap/iconTransparency"] = 0;
};

local listeners = {};

function HandleVariablesLoaded()
    SetCurrent(_G["yGather_Settings"]);
    _G["yGather_Settings"] = GetCurrent();
    SaveVariables("yGather_Settings");
end

function HandleSaveVariables()
    _G["yGather_Settings"] = GetCurrent();
    SaveVariables("yGather_Settings");
end

function SetValue(key, value)
    current[key] = value;
    Notify();
end

function GetValue(key)
    local value = temporary[key];
    if (value ~= nil) then
        return value;
    end

    value = current[key];
    if (value ~= nil) then
        return value;
    end

    return default[key];
end

function SetCurrent(newCurrent)
    if (newCurrent) then
        current = newCurrent;
        Notify();
    end
end

function GetCurrent()
    return current;
end

function Reset()
    current = {};
    temporary = {};
    Notify();
end

function SetTemporaryValue(key, value)
    temporary[key] = value;
    Notify();
end

function ApplyTemporary()
    for k, v in pairs(temporary) do
        current[k] = v;
    end
end

function ClearTemporary()
    temporary = {};
    Notify();
end

function AddListener(key, callback)
    listeners[key] = callback;
end

function RemoveListener(key)
    listeners[key] = nil;
end

function Notify()
    for key, callback in pairs(listeners) do
        callback(key);
    end
end
