if (yGather.worldmap == nil) then
    yGather.worldmap = {};
end

--[[ moves global icon references into more local context for faster access]]
function yGather.worldmap:InitializeIcons()
    local icons = {};
    self.icons = icons;
    for i = 0, 3 do
        for j = 0, 9 do
            for k = 0, 9 do
                local index = 100 * i + 10 * j + k
                icons[index + 1] = {
                    icon = getglobal("yGather_WorldmapResourceIcon" .. i .. j .. k),
                    texture = getglobal("yGather_WorldmapResourceIcon" .. i .. j .. k .. "_Texture"),
                }
                -- remove icon and textures from global table - not needed (there) anymore
                _G["yGather_WorldmapResourceIcon" .. i .. j .. k] = nil;
                _G["yGather_WorldmapResourceIcon" .. i .. j .. k .. "_Textures"] = nil;
            end
        end
    end
end

function yGather.worldmap:HideAllIcons()
    for _, icon in pairs(self.icons) do
        icon.icon:Hide();
    end
end

function yGather.worldmap:UpdateIcons()
    local zoneId = WorldMapFrame.mapID;
    
    self.statistics = nil;
    
    if (zoneId == nil) then
        yGather.logger.debug("[worldmap] zoneId = nil");
        self:HideAllIcons();
        return;
    end
    
    if (not yGather.settings.GetValue("worldmap/showResources")) then
        yGather.logger.debug("[worldmap] showing resources disabled");
        self:HideAllIcons();
        return;
    end

    local zoneData = yGather.database:GetZoneData(zoneId);
    if (zoneData == nil) then
        yGather.logger.debug("[worldmap] no data in zone: " .. zoneId);
        self:HideAllIcons();
        return;
    end

    yGather.logger.debug("[worldmap] showing resources in zone: " .. zoneId);
    
    self.updatedIcons = {};
    self.locationCount = 0;
    self.stacksFiltered = 0;
    local iconIndex = 1;
    for matId, resources in pairs(zoneData) do
        if (yGather_FilterPass(matId)) then
            iconIndex = self:ShowMaterialIcons(matId, resources, iconIndex);
        else
            self.stacksFiltered = self.stacksFiltered + #resources;
        end
        if (iconIndex > #self.icons) then
            break;
        end
    end

    yGather.logger.debug("[worldmap] setting tooltip");
    self.statistics = {
        stacksShown = self.locationCount,
        stacksFiltered = self.stacksFiltered,
        iconsShown = iconIndex - 1,
    }

    for i = iconIndex, #self.icons do
        self.icons[i].icon:Hide();
    end
end

function yGather.worldmap:ShowMaterialIcons(matId, resources, iconIndex)
    yGather.logger.debug("[worldmap] showing resources with id " .. matId);
    for i, coordinate in pairs(resources) do
        local x = coordinate.x;
        local y = coordinate.y;
        
        self.locationCount = self.locationCount + 1;
        
        local existingIcon = self:GetUpdatedIconAt(math.floor(x + 0.5), math.floor(y + 0.5), matId);
        
        if (existingIcon == nil) then
            self:ShowMaterialIconAt(x, y, matId, self.icons[iconIndex]);
            self.icons[iconIndex].icon.idx = i;
            iconIndex = iconIndex + 1;
            if (iconIndex > #self.icons) then
                break;
            end
        else
            existingIcon.name = existingIcon.name .. ", " .. yGather.resourceData:GetResourceName(matId);
            local skillLevel = yGather.resourceData:GetSkillLevel(matId);
            if ((skillLevel ~= nil) and (skillLevel > 0)) then
                existingIcon.name = existingIcon.name .. " (" .. skillLevel .. ")";
            end
            yGather.logger.debug("[worldmap] added resource to exising icon: " .. existingIcon.name);
        end;
    end
    return iconIndex;
end

function yGather.worldmap:GetUpdatedIconAt(x, y, matId)
    local skill = yGather.resourceData:GetResourceSkill(matId);
    local skillIcons = self.updatedIcons[skill];
    if (skillIcons == nil) then
        return nil;
    end
    
    local skillXIcons = skillIcons[x];
    if (skillXIcons == nil) then
        return nil;
    end
    
    return skillXIcons[y];
end

function yGather.worldmap:AddUpdatedIcon(skill, x, y, icon)
    
    if (self.updatedIcons[skill] == nil) then
        self.updatedIcons[skill] = {};
    end
    local skillIcons = self.updatedIcons[skill];
    
    if (skillIcons[x] == nil) then
        skillIcons[x] = {};
    end
    local skillXIcons = skillIcons[x];
    
    skillXIcons[y] = icon;
end

function yGather.worldmap:ShowMaterialIconAt(x, y, matId, iconData)
    local skill = yGather.resourceData:GetResourceSkill(matId);
    local textureFile = "Interface\\Icons\\" .. yGather.resourceData:GetResourceTexture(matId) .. ".dds";
    local icon = iconData.icon;
    local texture = iconData.texture;
    local frameWidth = yGather_WorldmapFrame:GetWidth();
    local frameHeight = yGather_WorldmapFrame:GetHeight();
    local iconSize = yGather.settings.GetValue("worldmap/iconSize");

    yGather.logger.debug("[worldmap] showing new icon: " .. matId .. " (" .. x .. ", " .. y .. ")");
    
    local name = yGather.resourceData:GetResourceName(matId);
    icon.name = name;
    local skillLevel = yGather.resourceData:GetSkillLevel(matId);
    if ((skillLevel ~= nil) and (skillLevel > 0)) then
        icon.name = icon.name .. " (" .. skillLevel .. ")";
    end
    icon.matid=matId;
    icon.zoneid = WorldMapFrame.mapID;
    self:ConfigureTexture(texture, textureFile, matId, skill);
    icon:ClearAllAnchors();
    icon:SetAnchor("CENTER", "TOPLEFT", "yGather_WorldmapFrame", x / 1000 * frameWidth, y / 1000 * frameHeight);
    icon:SetWidth(iconSize);
    icon:SetHeight(iconSize);
    icon:Show();
    yGather.logger.debug("showing on worldmap: " .. name .. " at (" .. x .. "," .. y .. ") ( " .. textureFile .. ")");
    self:AddUpdatedIcon(skill, math.floor(x + 0.5), math.floor(y + 0.5), icon);
end

function yGather.worldmap:ConfigureTexture(texture, textureFile, matid, skill)
    local showColoredDots = yGather.settings.GetValue("worldmap/iconMode") == "coloredDots";
    local transparency = yGather.settings.GetValue("worldmap/iconTransparency");

    local resourceInfo = yGather.resourceData:GetResourceInfo(matid);
    if (resourceInfo.skill == "PLAYER") then
        texture:SetTexture("interface\\minimap\\minimapicon.tga");
        texture:SetTexCoord(resourceInfo.textureIndex / 60.2, (resourceInfo.textureIndex + 1) / 60.2, 0, 1);
        texture:SetColor(resourceInfo.colorR, resourceInfo.colorG, resourceInfo.colorB, 1.0);
        texture:SetAlpha(1 - transparency);
    elseif (showColoredDots) then
        local color = self:GetColorForSkill(skill, matId);
        texture:SetTexture("interface\\minimap\\minimapicon.tga");
        texture:SetTexCoord(resourceInfo.textureIndex / 60.2, (resourceInfo.textureIndex + 1) / 60.2, 0, 1);
        texture:SetColor(color[1], color[2], color[3], 1.0);
        texture:SetAlpha(1 - transparency);
    else
        texture:SetTexture(textureFile);
        texture:SetTexCoord(0, 1, 0, 1);
        texture:SetColor(1.0, 1.0, 1.0, 1.0);
        texture:SetAlpha(1 - transparency);
    end
end

function yGather.worldmap:GetColorForSkill(skill, matId)
    if (skill == "LUMBERING") then 
        return yGather.settings.GetValue("worldmap/iconWoodColor");
    elseif (skill == "MINING") then
        return yGather.settings.GetValue("worldmap/iconOreColor");
    elseif (skill == "HERBLISM") then
        return yGather.settings.GetValue("worldmap/iconHerbsColor");
    else
        return yGather.resourceData:GetResourceColor(matId);
    end
end
