
yGather.database = {};
yGather.database.data = {}
yGather.database.listeners = {};
yGather.database.version = 1;

function yGather.database:HandleVariablesLoaded()
    local data = _G["yGather_data"];
    if (data == nil) then
        data = {version=self.version};
    end;
    
    while (data.version ~= self.version) do
        local oldVersion = data.version or 0;
        local convertFunctionFileName = "Interface/Addons/yGather/database/convert/v" .. oldVersion .. "Tov" .. (oldVersion + 1) .. ".lua";
        local convertFunction = dofile(convertFunctionFileName);
        data = convertFunction(data);
    end
    
    self.data = data;
    _G["yGather_data"] = self.data;
    SaveVariables("yGather_data");
    self:Notify();
end

function yGather.database:HandleSaveVariables()
    _G["yGather_data"] = self.data;
    SaveVariables("yGather_data");
end

function yGather.database:DumpZone(zoneId, zone)
    if (zone == nil) then
        zone = self:GetZoneData(zoneId);
    end
    for matId, mat in pairs(zone) do
        for locIdx, location in pairs(mat) do
            yGather.logger.user("[" .. zoneId .. "][" .. matId .. "][" .. locIdx .. "]= " .. location.x .. ","
                    .. location.y);
        end
    end
end

function yGather.database:Dump()
    for zoneId, zone in pairs(self.data) do
        yGather_DumpZone(zoneId, zone)
    end
end

function yGather.database:ClearZone(zone)
    if (zone == nil) then
        yGather.logger.info("clearing current zone");
        zone = yGather_ZONE;
    end
    self.data[zone] = {};
    self:Notify();
    yGather.logger.info("cleared zone " .. zone);
end

function yGather.database:GetStackCount()
    local stackCount = 0;
    for i, zone in pairs(self.data) do
        if (zone ~= nil) then
            for matId, stacks in pairs(zone) do
                if (stacks ~= nil) then
                    stackCount = stackCount + #stacks;
                end
            end
        end
    end
    return stackCount;
end

function yGather.database:GetZoneData(zone)
    if (zone == nil) then
        return nil;
    end;
    return self.data[zone];
end

function yGather.database:GetZoneResourceData(zone, resource)
    if (zone == nil) then
        return nil;
    end;
    if (resource == nil) then
        return nil;
    end;
    
    local zoneData = self.data[zone];
    if (zoneData == nil) then
        return nil;
    end
    
    return zoneData[resource];
end

function yGather.database:GetEntry(zone, material, index)
    local zoneData = self.data[zone];
    if (zoneData ~= nil) then
        local matData = zoneData[material];
        if (matData ~= nil) then
            return matData[index];
        end;
    end;
    return nil;
end

function yGather.database:EntryExists(zone, material, index)
    return (self:GetEntry(zone, material, index) ~= nil);
end

function yGather.database:GetEntryText(zone, material, index)
    local entry = self:GetEntry(zone, material, index);
    if (entry == nil) then
        return nil;
    end
    return entry.text;
end;

function yGather.database:SetEntryText(zone, material, index, text)
    local entry = self:GetEntry(zone, material, index);
    if (entry == nil) then
        error("no database entry for zone " .. zone .. ", material " .. material .. ", index " .. index, 1);
    end
    entry.text = text;
    self:Notify();
end;

function yGather.database:GetZoneStackCount(zoneId)
    if (zoneId == nil) then
        error("zoneId must not be nil", 2);
    end
    local stackCount = 0;
    local zone = self.data[zoneId];
    if (zone ~= nil) then
        for matId, stacks in pairs(zone) do
            if (stacks ~= nil) then
                stackCount = stackCount + #stacks;
            end
        end
    end
    return stackCount;
end

function yGather.database:RemoveEntry(zoneId, matId, index)
    if (zoneId == nil) then
        error("zoneId must not be nil", 2);
    end
    if (matId == nil) then
        error("matId must not be nil", 2);
    end
    if (index == nil) then
        error("index must not be nil", 2);
    end
    yGather.logger.debug("removing entry: " .. zoneId .. ", " .. matId .. ", " .. index);
    local zone = self.data[zoneId];
    if (zone ~= nil) then
        local locations = zone[matId];
        if (locations ~= nil) then
            locations[index] = nil;
            self:Notify();
            yGather.logger.debug("removed entry: " .. zoneId .. ", " .. matId .. ", " .. index);
        end
    end
end

function yGather.database:AddStackLocation(id, zone, x, y)
    if (id == nil) then
        error("id must not be nil", 2);
    end
    if (zone == nil) then
        error("zone must not be nil", 2);
    end
    if (x == nil) then
        error("id must not be nil", 2);
    end
    if (y == nil) then
        error("id must not be nil", 2);
    end

    yGather.logger.info("adding resource " .. id .. " in " .. zone .. " at (" .. x ..", " .. y .. ")");

    if (self.data[zone] == nil) then
        self.data[zone] = {};
    end
    if (self.data[zone][id] == nil) then
        self.data[zone][id] = {};
    end
    local zoneResourceData = self.data[zone][id];
    local newidx = table.maxn(zoneResourceData) + 1;

    zoneResourceData[newidx] = {};
    zoneResourceData[newidx].x = x;
    zoneResourceData[newidx].y = y;

    self:Notify();
    return newidx;
end

function yGather.database:AddListener(key, callback)
    self.listeners[key] = callback;
end

function yGather.database:RemoveListener(key)
    self.listeners[key] = nil;
end

function yGather.database:Notify()
    for key, callback in pairs(self.listeners) do
        callback(key);
    end
end
